/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "assist_imp_seldir.h"
#include "utilities.h"
#include "dialogscript.h"
#include "bible.h"
#include "utilities.h"
#include "usfm.h"
#include "usfmtools.h"
#include "dialogwait.h"


AssistantImportSelectDirectoryDialog::AssistantImportSelectDirectoryDialog(Gtk::Window& parent, AssistantPageType pagetype, const string& dir) :
  Dialog ("Import", parent, true, false)
{
  set_size_request(480, 320);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  if ((pagetype == ASSISTANT_PAGE_CONTENT) ||
    (pagetype == ASSISTANT_PAGE_CONFIRM))
    add_button(Gtk::Stock::GO_BACK, Gtk::RESPONSE_NO);
  if ((pagetype == ASSISTANT_PAGE_INTRO) ||
    (pagetype == ASSISTANT_PAGE_CONTENT))
    nextbutton = add_button(Gtk::Stock::GO_FORWARD, Gtk::RESPONSE_YES);
  if (pagetype == ASSISTANT_PAGE_CONFIRM)
    add_button(Gtk::Stock::APPLY, Gtk::RESPONSE_OK);
  label1.set_text("Select the directory that has the files to import");
  label1.set_padding(0, 10);
  get_vbox()->pack_start(label1, 0, 0, 10);
  if (dir.empty())
    directory = Glib::get_home_dir();
  else
    directory = dir;
  button.signal_clicked().connect(sigc::mem_fun(*this, &AssistantImportSelectDirectoryDialog::on_button));
  get_vbox()->pack_start(button, 0, 0, 10);
  label2.set_padding(0, 10);
  get_vbox()->pack_start(label2, 0, 0, 10);
    
  if (pagetype == ASSISTANT_PAGE_CONFIRM)
    set_default_response(Gtk::RESPONSE_OK);
  else 
    set_default_response(Gtk::RESPONSE_YES);
  set_gui();
  show_all_children();
}


AssistantImportSelectDirectoryDialog::~AssistantImportSelectDirectoryDialog() {
}


void AssistantImportSelectDirectoryDialog::on_button() {
  int result;
  string mydir;
  {
    Gtk::FileSelection dialog("Please choose a folder");
    dialog.set_transient_for(*this);
    dialog.get_file_list()->get_parent()->hide(); //Prevent the user from selecting a file.
    dialog.hide_fileop_buttons();	// Hide the buttons that operate on files.
    dialog.set_filename(directory + "/");
    result = dialog.run();
    mydir = dialog.get_filename();
  }
  if (result == Gtk::RESPONSE_OK) {  //Handle the response:
    WaitDialog wd (*this, 100, 0, 0);
    wd.run();
    directory = mydir;
    sfms.clear(); // Clear any previous data.
    set_gui();
  }
}


void AssistantImportSelectDirectoryDialog::set_gui() {
  button.set_label (directory);
  // Read directory, and leave subdirectories out.
  vector<string> filenames;
  {
    ReadFiles rd (directory, "", "");
    for (unsigned int i = 0; i < rd.files.size(); i++) {
     string filename = Glib::build_filename(directory, rd.files[i]);
  	    filenames.push_back(filename);
    }
  }
  // See which files seem to be the SFM ones.
  for (unsigned int i = 0; i < filenames.size(); i++)
    if (!get_id_from_file(filenames[i]).empty())
      sfms.push_back(filenames[i]);
  // Info for user.
  if (filenames.size() == 0) {
  	label2.set_text ("This directory contains no files");
  } else {
    string info = "Number of files in this directory: " + convert_to_string(int(filenames.size()));
    info.append("\nNumber of files that appear to be SFM texts: " + convert_to_string(int(sfms.size())));
    label2.set_text (info);
  }
  // We only can go forward if we've found sfm texts.
  nextbutton->set_sensitive(sfms.size() > 0);
}


