/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "bible.h"
#include "utilities.h"


char *USFM_IDs[NUMBER_OF_BIBLEBOOKS] = 
  {"GEN", "EXO", "LEV", "NUM", "DEU", "JOS", "JDG", "RUT", "1SA", "2SA", "1KI", "2KI", "1CH", "2CH", "EZR", "NEH", "EST", "JOB", "PSA", "PRO", 
   "ECC", "SNG", "ISA", "JER", "LAM", "EZK", "DAN", "HOS", "JOL", "AMO", "OBA", "JON", "MIC", "NAM", "HAB", "ZEP", "HAG", "ZEC", "MAL",
   "MAT", "MRK", "LUK", "JHN", "ACT", "ROM", "1CO", "2CO", "GAL", "EPH", "PHP", "COL", "1TH", "2TH", "1TI", "2TI", "TIT", "PHM", "HEB", "JAS", 
   "1PE", "2PE", "1JN", "2JN", "3JN", "JUD", "REV"};
char *BibleWorksIDs[NUMBER_OF_BIBLEBOOKS] =
  {"Gen", "Exo", "Lev", "Num", "Deu", "Jos", "Jdg", "Rut", "1Sa", "2Sa", "1Ki", "2Ki", "1Ch", "2Ch", "Ezr", "Neh", "Est", "Job", "Psa", "Pro", 
   "Ecc", "Sol", "Isa", "Jer", "Lam", "Eze", "Dan", "Hos", "Joe", "Amo", "Oba", "Jon", "Mic", "Nah", "Hab", "Zep", "Hag", "Zec", "Mal",
   "Mat", "Mar", "Luk", "Joh", "Act", "Rom", "1Co", "2Co", "Gal", "Eph", "Phi", "Col", "1Th", "2Th", "1Ti", "2Ti", "Tit", "Phm", "Heb", "Jam", 
   "1Pe", "2Pe", "1Jo", "2Jo", "3Jo", "Jud", "Rev"};
char *OSIS_IDs[NUMBER_OF_BIBLEBOOKS] =
  {"Gen", "Exod", "Lev", "Num", "Deut", "Josh", "Judg", "Ruth", "1Sam", "2Sam", "1Kgs", "2Kgs", "1Chr", "2Chr", "Ezra", "Neh", "Esth", "Job",
   "Ps", "Prov", "Eccl", "Song", "Isa", "Jer", "Lam", "Ezek", "Dan", "Hos", "Joel", "Amos", "Obad", "Jonah", "Mic", "Nah", "Hab", "Zeph",
   "Hag", "Zech", "Mal", 
   "Matt", "Mark", "Luke", "John", "Acts", "Rom", "1Cor", "2Cor", "Gal", "Eph", "Phil", "Col", "1Thess", "2Thess", "1Tim", "2Tim", "Titus",
   "Phlm", "Heb", "Jas", "1Pet", "2Pet", "1John", "2John", "3John", "Jude", "Rev"};
char *EnglishBookNames[NUMBER_OF_BIBLEBOOKS] =
  {"Genesis", "Exodus", "Leviticus", "Numbers", "Deuteronomy", "Joshua", "Judges", "Ruth", "1 Samuel", "2 Samuel", "1 Kings", "2 Kings",
   "1 Chronicles", "2 Chronicles", "Ezra", "Nehemiah", "Esther", "Job", "Psalms", "Proverbs", "Ecclesiastes", "Song of Solomon", "Isaiah", 
   "Jeremiah", "Lamentations", "Ezekiel", "Daniel", "Hosea", "Joel", "Amos", "Obadiah", "Jonah", "Micah", "Nahum", "Habakkuk", "Zephaniah",
   "Haggai", "Zechariah", "Malachi",
   "Matthew", "Mark", "Luke", "John", "Acts", "Romans", "1 Corinthians", "2 Corinthians", "Galatians", "Ephesians", "Philippians", "Colossians",
   "1 Thessalonians", "2 Thessalonians", "1 Timothy", "2 Timothy", "Titus", "Philemon", "Hebrews", "James", "1 Peter", "2 Peter", "1 John",
   "2 John", "3 John", "Jude", "Revelation"};

string id_to_biblebook_english (const string& id) {
  // Takes id and returns an English bookname.
  // E.g. GEN returns "Genesis".
  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
    if (id == USFM_IDs[i])
      return EnglishBookNames[i];
  }
  return "Unknown";
}


string index_to_paratext_id (unsigned int index) {
  return USFM_IDs[index];
}


unsigned paratext_id_to_index (const string& id) {
  // Returns the index of the Paratext ID. So e.g. EXO returns 1, and GEN 0
  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
    if (id == USFM_IDs[i]) 
      return i;
  }
  return string::npos;
}


unsigned bibleworks_id_to_index (const string& id) {
  // Returns the index of the BibleWorks ID. So e.g. Exo returns 1, and Gen 0
  string id2 = upperCase(id);
  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
    if (id2 == upperCase(BibleWorksIDs[i])) 
      return i;
  }
  return string::npos;
}


unsigned osis_id_to_index (const string& id) {
  // Returns the index of the OSIS ID. So e.g. Exod returns 1, and Gen 0
  string id2 = upperCase(id);
  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
    if (id2 == upperCase(OSIS_IDs[i])) 
      return i;
  }
  return string::npos;
}


unsigned english_id_to_index (const string& id) {
  // Returns the index of the English name, e.g. "1Corinthians" or "1 Corinthians"
  string id2 = upperCase(id);
  for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
    if (id2 == upperCase(EnglishBookNames[i])) 
      return i;
  }
  return string::npos;
}


void sort_references (vector<string>& references) {
  /*
    Sorts all references from Genesis to Revelation. 
    Sorts on book first, then chapter, and finally verse.
  */
  // When there are less than two entries no sorting is needed.
  if (references.size() < 2) return;
  try {
    // Make a vector parallel to "references", with only the
    // English book name in it.
    vector<string> english;
    for (unsigned int i = 0; i < references.size(); i++) {
      int bookstop;
      bookstop = references[i].find(" ", 2);
      english.push_back(references[i].substr(0, bookstop));
    }
    vector<string> sorted;
    for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
      string bookname = EnglishBookNames[i];
      for (unsigned int i2 = 0; i2 < references.size(); i2++) {
        if (bookname == english[i2])
	    sorted.push_back(references[i2]);
      }
    }
    references.clear();
    for (unsigned int i = 0; i < sorted.size(); i++)
      references.push_back(sorted[i]);
  }
  catch (exception& ex) {
    cerr << "Sorting references: " << ex.what() << endl;
  }
  /*
    In theory this simple procedure is not sufficient,
    because it only sorts on the books, not on chapters and verses.
    However, in practice we see that the references are sorted.
    Most likely this is caused by the database that presents
    its results in this order.
    So we leave it just now, until practice shows the need 
    for a better sorting mechanism.
    This would consist of taking these extra steps:
    - Get the data book by book (as we already have)
    - Extract the chapter:verse part.
    - Send this part through command "sort -n <file>.
    - The output will be sorted on chapter and verse.
    - Rejoin the sorted output with the original book
   */
}


string index_to_biblebook (unsigned int index) {
  return EnglishBookNames[index];  
}
