/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "worddatabase.h"
#include "scripture.h"
#include "book.h"
#include "utilities.h"
#include "bible.h"


int main(int argc, char *argv[]) {
/*
 * This program produces and updates the database with all the words in a project.
 * At first it was integrated in the bibledit.bin program, using threads to do
 * the time consuming jobs. But when the object doing this was destroyed, 
 * either because the project was closed or the program was stopped, the thread
 * caused trouble. There were some exceptions generated by sqlite that could 
 * not be caught for some reason. So bibledit regularly crashed. The solution
 * is now to let this separate console program do all the things related
 * to the word database. This program will be run by the shell and is independent
 * of bibledit, so it can never crash bibledit.
 * 
 * It can be called in three ways:
 * 1. bibledit.wdb --initialize <projectname>
 * 2. bibledit.wdb --update <projectname> <bookname> <chapternumber>
 * 3. bibledit.wdb --finalize <projectname>
 */
  string arg1 = argv[1];
  string project = argv[2];
  Worddatabase db(project);
  if (arg1 == "--initialize") {
    db.initialize();
  } else if (arg1 == "--update") {
  	string bookname = argv[3];
  	Scripture scripture(project);
    int index = -1;
  	for (unsigned int i = 0; i < scripture.books.size(); i++) {
      if (bookname == scripture.books[i]) {
      	index = i;
      	break;
      }
  	}
  	Book book(scripture.paths[index], false);
	unsigned int chapternumber = convert_to_int(argv[4]);
    vector<Glib::ustring> lines;
    book.get_chapter(chapternumber, lines);
    db.update_chapter(english_id_to_index(bookname), chapternumber, lines);
  } else if (arg1 == "--finalize") {
    // Vacuum, that is, optimize, the words database.
    string command = "echo \"vacuum;\" | sqlite3 '" + db.dbfilename() + "'";
    system(command.c_str());
  }
  return 0;  
}

