/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "configuration.h"
#include "utilities.h"
#include "config.h"
#include "glibmm.h"
#include "directories.h"


string synchronization_paths_filename() {
  return Glib::build_filename (directories_get_root(), "synchronization-paths");
}


Configuration::Configuration (Gtk::Window& window) {
  // Read the configuration file, if there is any. Add the bits that are missing.
  ReadText rc(bibleditrc(), true);
  screen_resolution_changed = false;
  for (unsigned int i = 0; i <= 1000; i++) {
    switch (i) {
    case 2: 
      {
	if (i < rc.lines.size()) { 
	  projectdirectory_reuse = rc.lines[i];
	} else { 
	  projectdirectory_reuse = Glib::path_get_dirname(bibleditrc()); 
	}
	break;
      }
    case 3:
      { 
	if (i < rc.lines.size()) {
	  projectshortcut_reuse = rc.lines[i];
	} else { 
	  projectshortcut_reuse.clear(); 
	}
	break; 
      }
    case 4:
      {
	if (i < rc.lines.size()) { 
	  converttowebdirectory = rc.lines[i];
	} else { 
	  converttowebdirectory = Glib::path_get_dirname(bibleditrc()) + "/bibledit-web-output";
	} 
	break;
      }
    case 5:
      {
	if (i < rc.lines.size()) { 
	  screen_width = convert_to_int (rc.lines[i]);
	} else { 
	  screen_width = 0;
	} 
	if (screen_width != window.get_screen()->get_width()) {
	  screen_resolution_changed = true;
	  screen_width = window.get_screen()->get_width();
	}
	break;
      }
    case 6:
      {
	if (i < rc.lines.size()) { 
	  screen_height = convert_to_int (rc.lines[i]);
	} else { 
	  screen_height = 0;
	} 
	if (screen_height != window.get_screen()->get_height()) {
	  screen_resolution_changed = true;
	  screen_height = window.get_screen()->get_height();
	}
	break;
      }
    case 7:
      {
	bool take_default = false;
	if (i < rc.lines.size()) {
	  window_width = convert_to_int (rc.lines[i]);
	} else { 
	  take_default = true;
	} 
	if (screen_resolution_changed || take_default) {
	  window_width = screen_width * 95 / 100;
	}
	break;
      }
    case 8:
      {
	bool take_default = false;
	if (i < rc.lines.size()) {
	  window_height = convert_to_int (rc.lines[i]);
	} else { 
	  take_default = true;
	} 
	if (screen_resolution_changed || take_default) {
	  window_height = screen_height * 90 / 100;
	}
	break;
      }
    case 9:
      {
	bool take_default = false;
	if (i < rc.lines.size()) {
	  window_x_position = convert_to_int (rc.lines[i]);
	} else { 
	  take_default = true;
	} 
	if (screen_resolution_changed || take_default) {
	  window_x_position = 0;
	}
	break;
      }
    case 10:
      {
	bool take_default = false;
	if (i < rc.lines.size()) {
	  window_y_position = convert_to_int (rc.lines[i]);
	} else { 
	  take_default = true;
	} 
	if (screen_resolution_changed || take_default) {
	  window_y_position = 0;
	}
	break;
      }
    case 11:
      {
	bool take_default = false;
	if (i < rc.lines.size()) {
	  hpane_position = convert_to_int (rc.lines[i]);
	} else { 
	  take_default = true;
	} 
	if (screen_resolution_changed || take_default) {
	  hpane_position = window_width * 87 / 100;
	}
	break;
      } 
    case 12:
      {
	bool take_default = false;
	if (i < rc.lines.size()) {
	  vpane_position = convert_to_int (rc.lines[i]);
	} else { 
	  take_default = true;
	} 
	if (screen_resolution_changed || take_default) {
	  vpane_position = window_height * 50 / 100;
	}
	break;
      }
    case 13:
      {
	if (i < rc.lines.size()) { 
	  book_opened = rc.lines[i];
	} else { 
	  book_opened.clear();
	} 
	break;
      }
    case 14:
      {
	if (i < rc.lines.size()) { 
	  chapter_opened = rc.lines[i];
	} else { 
	  chapter_opened = "1";
	} 
	break;
      }
    case 15:
      {
	if (i < rc.lines.size()) { 
	  verse_opened = rc.lines[i];
	} else { 
	  verse_opened = "1";
	} 
	break;
      }

    case 16:
      {
	if (i < rc.lines.size()) { 
	  synchronize_paratext_projectdirectory = rc.lines[i];
	} else { 
	  synchronize_paratext_projectdirectory.clear();
	} 
	break;
      }

    case 17:
      {
	if (i < rc.lines.size()) { 
	  synchronize_paratext = convert_to_bool(rc.lines[i]);
	} else { 
	  synchronize_paratext = false;
	} 
	break;
      }

    case 18:
      {
	if (i < rc.lines.size()) { 
	  find_searchword = rc.lines[i];
	} else { 
	  find_searchword.clear();
	} 
	break;
      }

    case 19:
      {
	if (i < rc.lines.size()) { 
	  find_replaceword = rc.lines[i];
	} else { 
	  find_replaceword.clear();
	} 
	break;
      }

    case 20:
      {
	if (i < rc.lines.size()) { 
	  references_file = rc.lines[i];
	} else { 
	  references_file.clear();
	} 
	break;
      }

    case 21:
      {
	if (i < rc.lines.size()) { 
	  companion_local_enabled = convert_to_bool(rc.lines[i]);
	} else { 
	  companion_local_enabled = false;
	} 
	break;
      }

    case 22:
      {
	if (i < rc.lines.size()) { 
	  companion_remote_enabled = convert_to_bool(rc.lines[i]);
	} else { 
	  companion_remote_enabled = false;
	} 
	break;
      }

    case 23:
      {
	if (i < rc.lines.size()) { 
	  companion_remote_name = rc.lines[i];
	} else { 
	  companion_remote_name.clear();
	} 
	break;
      }

    case 24:
      {
	if (i < rc.lines.size()) { 
	  editor_font_name = rc.lines[i];
	} else { 
	  editor_font_name = "Sans 14";
	} 
	break;
      }

    case 25:
      {
	if (i < rc.lines.size()) { 
	  last_goto_reference = rc.lines[i];
	} else { 
	  last_goto_reference.clear();
	} 
	break;
      }

    case 26:
      {
	if (i < rc.lines.size()) { 
	  project = rc.lines[i];
	} else { 
	  project.clear();
	} 
	break;
      }

    }
  }
}


Configuration::~Configuration () {
  // If the object goes out of scope, it saves the configuration.
  vector<string> output;
  output.push_back(VERSION);
  output.push_back("There is no need for editing this file, because it is maintained by bibledit");
  output.push_back(projectdirectory_reuse);
  output.push_back(projectshortcut_reuse);
  output.push_back(converttowebdirectory);
  output.push_back(convert_to_string(screen_width));
  output.push_back(convert_to_string(screen_height));
  output.push_back(convert_to_string(window_width));
  output.push_back(convert_to_string(window_height));
  output.push_back(convert_to_string(window_x_position));
  output.push_back(convert_to_string(window_y_position));
  output.push_back(convert_to_string(hpane_position));
  output.push_back(convert_to_string(vpane_position));
  output.push_back(book_opened);
  output.push_back(chapter_opened);
  output.push_back(verse_opened);
  output.push_back(synchronize_paratext_projectdirectory);
  output.push_back(convert_to_string(synchronize_paratext));
  output.push_back(find_searchword);
  output.push_back(find_replaceword);
  output.push_back(references_file);
  output.push_back(convert_to_string(companion_local_enabled));
  output.push_back(convert_to_string(companion_remote_enabled));
  output.push_back(companion_remote_name);
  output.push_back(editor_font_name);
  output.push_back(last_goto_reference);
  output.push_back(project);
  write_lines(bibleditrc(), output);
}


string Configuration::bibleditrc () {
  // Returns the full name of the ~/<root>/settings configuration file.
  // E.g.: /home/joe/.bibledit/settings
  string bibledit_directory = directories_get_root();
  return Glib::build_filename (bibledit_directory, "settings");
}


