/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogcompanion.h"
#include "utilities.h"


CompanionDialog::CompanionDialog(Gtk::Window& parent, Configuration& configuration) :
  Dialog ("Bibledit Windows Companions", parent, true, false)
{
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

  label1.set_text("Bibledit can work together with a \"Bibledit Windows Companion\".\n" \
		  "This companion gives Bibledit access to some Bible translation programs\n" \
		  "that run on Windows or Wine. There can be two companions.\n" \
		  "\n" \
		  "One running on this computer. It communicates with Windows programs running on Wine.");
  label1.set_padding(5, 5);
  get_vbox()->pack_start(label1, false, false, 0);

  get_vbox()->pack_start(checkbuttonlocal, false, false, 5);
  checkbuttonlocal.set_label("Use the companion on this computer");
  checkbuttonlocal.set_active(configuration.companion_local_enabled);

  label2.set_text("\nAnother one running on a virtual machine with Windows programs running there.");
  label2.set_padding(5, 5);
  label2.set_alignment(0);
  get_vbox()->pack_start(label2, false, false, 0);

  get_vbox()->pack_start(checkbuttonremote, false, false, 5);
  checkbuttonremote.set_label("Use the companion on the virtual machine");
  checkbuttonremote.set_active(configuration.companion_remote_enabled);
  checkbuttonremote.signal_clicked().connect(sigc::mem_fun(*this, &CompanionDialog::set_gui));

  label3.set_text("Enter its IP address or hostname below.");
  label3.set_padding(5, 5);
  label3.set_alignment(0);
  get_vbox()->pack_start(label3, false, false, 0);

  entry.set_text(configuration.companion_remote_name);
  get_vbox()->pack_start(entry, false, false, 0);

  set_gui();
  show_all_children();

  // Focus is on the checkbutton. Convenient for the user.
  set_focus_child(checkbuttonlocal);
}


CompanionDialog::~CompanionDialog() {
}


void CompanionDialog::set_gui() {
  label3.set_sensitive(checkbuttonremote.get_active());
  entry.set_sensitive(checkbuttonremote.get_active());
}

