/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogconvert2web.h"
#include "utilities.h"
#include "constants.h"
#include "dialogscript.h"
#include "htmlbrowser.h"
#include "html.h"
#include "scripture.h"


ConvertToWebDialog::ConvertToWebDialog(Gtk::Window& parent, Configuration& configuration) :
  Dialog ("Convert a project to web files", parent, true, false),
  button2("Convert"),
  button3("Open in browser")
{
  converttowebdirectory = configuration.converttowebdirectory;

  // Store all the filenames to process.
  Scripture scripture(configuration.project);
  for (unsigned int i = 0; i < scripture.paths.size(); i++) {
    filenames.push_back(scripture.paths[i]);
  }

  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

  label1.set_text("Converting a project to web files consist of the following steps.\n" \
		 "1. Indicating the directory where the web files are to be placed.\n" \
		 "   (this is not needed every time, only when there is a change)\n" \
		 "2. Converting the project.");
  label1.set_padding(10, 10);
  get_vbox()->pack_start(label1, false, false, 0);

  button1.signal_clicked().connect(sigc::mem_fun(*this, &ConvertToWebDialog::on_button1_clicked));
  get_vbox()->pack_start(button1, false, false, 10);

  button2.signal_clicked().connect(sigc::mem_fun(*this, &ConvertToWebDialog::on_button2_clicked));
  get_vbox()->pack_start(button2, false, false, 10);

  button3.signal_clicked().connect(sigc::mem_fun(*this, &ConvertToWebDialog::on_button3_clicked));
  get_vbox()->pack_start(button3, false, false, 10);

  set_gui();
  show_all_children();
  button2.grab_focus();
}


ConvertToWebDialog::~ConvertToWebDialog() {
}


void ConvertToWebDialog::on_button1_clicked() {
  Gtk::FileSelection dialog("Please choose a folder");
  dialog.set_transient_for(*this);
  dialog.get_file_list()->get_parent()->hide(); //Prevent the user from selecting a file.
  dialog.hide_fileop_buttons();	// Hide the buttons that operate on files.
  dialog.set_filename(converttowebdirectory + "/");
  int result = dialog.run();
  if (result == Gtk::RESPONSE_OK) {  //Handle the response:
    converttowebdirectory = dialog.get_filename();
    set_gui();
  }
}


void ConvertToWebDialog::on_button2_clicked() {
  HtmlConverter hc (0);
  hc.convert (filenames, converttowebdirectory);
}


void ConvertToWebDialog::on_button3_clicked() {
  htmlbrowser (*this, Glib::build_filename(converttowebdirectory, "index.html"));
}


void ConvertToWebDialog:: set_gui() {
  // This sets the labels etc. of the dialog.
  button1.set_label("  Indicate directory for the web files: " + converttowebdirectory + "  ");
}



