/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogfind.h"
#include "utilities.h"
#include "dialogscript.h"
#include "sqlite3_plus.h"
#include "bible.h"
#include "utilities.h"


FindDialog::FindDialog(Gtk::Window& parent, const string& database) :
  Dialog ("Find", parent, true, false)
{
  findbutton = add_button(Gtk::Stock::FIND, Gtk::RESPONSE_OK);
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  // Set up all the widgets
  vbox.pack_start(hbox);
  hbox.pack_start(label, 0, 0, 10);
  label.set_text("Search for");
  label.set_padding(0, 10);
  hbox.pack_start(entry, 0, 0, 10);
  get_vbox()->pack_start(vbox);
  // Focus the entry
  set_focus_child(entry);
  // Default response button.
  findbutton->set_flags(Gtk::CAN_DEFAULT);
  findbutton->grab_default();
  set_default_response(Gtk::RESPONSE_OK);
  show_all_children();
  mydatabase = database;
}


FindDialog::~FindDialog() {
}


void FindDialog::on_response (int response_id) {
  if (response_id == Gtk::RESPONSE_OK) {
    try {
      // Connect to database.
      sqlite3::connection con(mydatabase.c_str());
      // Get the string to search for. Note any apostrophies need to be doubled for SQLite.
      // We need to normalize the search expression, as prescribed, when comparing strings.
      Glib::ustring searchword;
      searchword = entry.get_text().normalize();
      searchword = searchword.casefold();
      searchword = double_apostrophy(searchword);
      // Execute the SQL statement, and read the results.
      sqlite3::reader reader = con.executereader("select book, chapter, verse from 'line_casefolded' where line glob ('*%s*');", searchword.c_str());
      while(reader.read()) {
        results.push_back(index_to_biblebook (reader.getint32(0)) + " " + reader.getstring(1) + ":" + reader.getstring(2));
      }
      // Close all
      reader.close();
      con.close();
    }
    catch(exception &ex) {
      cerr << "Error: " << ex.what() << endl;
    }
  }
}


