/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogimport.h"
#include "assist_info.h"
#include "assist_info2.h"
#include "assist_imp_seldir.h"
#include "assist_imp_sfm.h"
#include "usfmtools.h"
#include "bible.h"
#include "scripture.h"
#include "utilities.h"
#include "dialogwait.h"
#include "utilities.h"
#include "book.h"
#include "projectutils.h"
#include <set>
#include "usfmtools.h"


#define TEMP_DIR "bibledit.import"


ImportDialog::ImportDialog(Gtk::Window& parent, const string& project) :
  Dialog ("Import", parent, true, false)
{
  // Variables
  myproject = project;
  // Set up all the widgets
  add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);
  label.set_text("You are going to import text into the project");
  label.set_padding(0, 10);
  get_vbox()->pack_start(label, 0, 0, 10);
  button1.set_label("Import files with Standard Format Markers");
  button1.signal_clicked().connect(sigc::mem_fun(*this, &ImportDialog::on_sfm));
  get_vbox()->pack_start(button1, 0, 0, 10);
  set_default_response(Gtk::RESPONSE_OK);
  show_all_children();
  // Make the temporary director where we're going to place the files to import.
  create_directory(temporary_file(TEMP_DIR));
}


ImportDialog::~ImportDialog() {
  // Clean up temporary files.
  string command = "rm -rf '";
  command.append(temporary_file(TEMP_DIR));
  command.append ("'");
  Shell shell (command);
  shell.run();
}


void ImportDialog::on_sfm() {
  int dialognumber = 0;
  string directory;
  vector<string> sfmfiles;
  while (true) {
    switch (dialognumber) {
      case 0: {
        string message = ("You are going to import Standard Format Marker texts\n" \
                   "into your project");
        AssistantInfoDialog dialog(*this, "Import", ASSISTANT_PAGE_INTRO, message);
        int result = dialog.run();
        switch(result) {
          case Gtk::RESPONSE_YES : {
            dialognumber++;
            break;
          }
          default : {
            response(Gtk::RESPONSE_CANCEL);
          	return;
          }
        }
        break;
      }

      case 1: {
        WaitDialog wd (*this, 500, 480, 320);
        wd.run();
        AssistantImportSelectDirectoryDialog dialog(*this, ASSISTANT_PAGE_CONTENT, directory);
        int result = dialog.run();
        directory = dialog.directory;
        sfmfiles.clear();
        for (unsigned int i = 0; i < dialog.sfms.size(); i++)
          sfmfiles.push_back(dialog.sfms[i]);
        switch (result) {
          case Gtk::RESPONSE_YES : {
            dialognumber++;
            break;
          }
          case Gtk::RESPONSE_NO : {
            dialognumber--;
            break;
          }
          default : {
            response(Gtk::RESPONSE_CANCEL);
          	return;
          }
        }
        break;
      }

      case 2: {
        vector<string> message;
        message.push_back("Here is the list of files that will be used to import text from:");
        message.push_back("");
      	for (unsigned int i = 0; i < sfmfiles.size(); i++) {
      	  message.push_back (sfmfiles[i]);
      	}
        message.push_back("");
        message.push_back("If you are not satisfied with it, add some files to the directory, or remove some.");
        AssistantInfo2Dialog dialog(*this, "Import", ASSISTANT_PAGE_CONTENT, message, true);
        int result = dialog.run();
        switch(result) {
          case Gtk::RESPONSE_YES : {
            dialognumber++;
            break;
          }
          case Gtk::RESPONSE_NO : {
            dialognumber--;
            break;
          }
          default : {
            response(Gtk::RESPONSE_CANCEL);
          	return;
          }
        }
        break;
      }

      case 3: {
        // Some visual candy...
        WaitDialog wd (*this, 200, 480, 320);
        wd.run();
        // Clear any previously copied files.
        {
          string command = "rm -rf ";
          command.append(temporary_file(TEMP_DIR));
          command.append("/*");
          Shell shell (command);
          shell.run();
        }
        // This variabele shows whether we've some books.
        bool there_are_books_to_import = false;
        // Make a vector that contains all names of the smsfiles.
        vector<string> smsfiles_names;
        for (unsigned int i = 0; i < sfmfiles.size(); i++) {
          smsfiles_names.push_back(id_to_biblebook_english(get_id_from_file(sfmfiles[i])));
        }
        vector<string> message;
        Scripture scripture(myproject);
        for (unsigned int i = 0; i < NUMBER_OF_BIBLEBOOKS; i++) {
          string bookname = index_to_biblebook(i);
          message.push_back(bookname);
          bool inproject = false;
          for (unsigned int i2 = 0; i2 < scripture.books.size(); i2++) {
            if (scripture.books[i2] == bookname) {
              inproject = true;
              break;
            }
          }
          if (inproject) {
            message.push_back(" - already in project ... skipping");
          } else {
            // Search if the book is among the files.
            vector<string> filenames_particular_book;
            for (unsigned int i2 = 0; i2 < sfmfiles.size(); i2++) {
              if (smsfiles_names[i2] == bookname) {
            	message.push_back("- we found " + sfmfiles[i2]);
            	filenames_particular_book.push_back(sfmfiles[i2]);
              }
            }
            // Sort out what we found.
            string booktocopy;
            if (filenames_particular_book.empty()) {
              message.push_back("- could not be found among the files");
            } else if (filenames_particular_book.size() == 1) {
              booktocopy = filenames_particular_book[0];
              message.push_back("- we pick " + booktocopy);
            } else {
              booktocopy = pick_newest_filename(filenames_particular_book);
              message.push_back("- we take the newest: " + booktocopy);
            }
            // If we've a book, copy it now.
            if (!booktocopy.empty()) {
              string command = "cp '";
              command.append(booktocopy);
              command.append("' '");
              command.append(temporary_file(TEMP_DIR));
              command.append("/" + bookname);
              command.append("'");
              Shell shell (command);
              shell.run();
              there_are_books_to_import = true;
            }
          }
        }
        if (!there_are_books_to_import) {
          message.push_back("");
          message.push_back("There are no books that can be imported");
        }
        AssistantInfo2Dialog dialog(*this, "Import", ASSISTANT_PAGE_CONTENT, message, there_are_books_to_import);
        int result = dialog.run();
        switch(result) {
          case Gtk::RESPONSE_YES : {
            dialognumber++;
            break;
          }
          case Gtk::RESPONSE_NO : {
            dialognumber--;
            break;
          }
          default : {
            response(Gtk::RESPONSE_CANCEL);
          	return;
          }
        }
        break;
      }

      case 4: {
        // Final assistant dialog.
        string message = 
          ("We now know which files to import.\n" \
           "The next thing is to check if the files are in Unicode format.\n" \
           "\n" \
           "If a file is found that is not in Unicode format,\n" \
           "you will be asked to convert it.\n" \
           "\n" \
           "Note: The conversion is done once for every book, so you may be asked\n" \
           "several times, depending on how many books you import.");
        AssistantInfoDialog dialog(*this, "Import", ASSISTANT_PAGE_CONTENT, message);
        int result = dialog.run();
        switch (result) {
          case Gtk::RESPONSE_YES : {
            dialognumber++;
            break;
          }
          case Gtk::RESPONSE_NO : {
            dialognumber--;
            break;
          }
          default : {
            response(Gtk::RESPONSE_CANCEL);
          	return;
          }
        }
        break;
      }

      case 5: {
        // Some visual candy, because the process might take time.
        WaitDialog wd (*this, 200, 480, 320);
        wd.run();
        // Check whether everything is in Unicode.
        string directory = temporary_file(TEMP_DIR);
        ReadFiles rd (directory, "", "");
        for (unsigned int i = 0; i < rd.files.size(); i++) {
          string filename = Glib::build_filename(directory, rd.files[i]);
          Book book(*this, filename, false);
        }
        string message = ("Everything is now in Unicode format.\n" \
                          "\n" \
                          "Next you are given an opportunity to adjust\n" \
                          "the Standard Format Markers.");
        AssistantInfoDialog dialog(*this, "Import", ASSISTANT_PAGE_CONTENT, message);
        int result = dialog.run();
        switch (result) {
          case Gtk::RESPONSE_YES : {
            dialognumber++;
            break;
          }
          case Gtk::RESPONSE_NO : {
            dialognumber--;
            break;
          }
          default : {
            response(Gtk::RESPONSE_CANCEL);
          	return;
          }
        }
        break;
      }

      case 6: {
        // Some eye candy, because the process might take time.
        WaitDialog wd (*this, 200, 480, 320);
        wd.run();
        // Collect all SFM's and decide how we deal with them.
        set<string> sfms;
        string directory = temporary_file(TEMP_DIR);
        ReadFiles rd (directory, "", "");
        for (unsigned int i = 0; i < rd.files.size(); i++) {
          string filename = Glib::build_filename(directory, rd.files[i]);
          if (Glib::file_test(filename, Glib::FILE_TEST_IS_REGULAR)) {
            Book book(filename, true);
            for (unsigned int j = 0; j < book.number_of_chapters; j++) {
              vector<Glib::ustring> lines;
              book.get_chapter(j + 1, lines);
              for (unsigned int k = 0; k < lines.size(); k++) {
                Glib::ustring line = lines[k];
                Glib::ustring marker = usfm_extract_marker(line);
                sfms.insert(marker);
              }
            }
          }
        }
        AssistantImportSFMDialog dialog(*this, "Import", ASSISTANT_PAGE_CONTENT, sfms);
        int result = dialog.run();
        switch (result) {
          case Gtk::RESPONSE_YES : {
            dialognumber++;
            break;
          }
          case Gtk::RESPONSE_NO : {
            dialognumber--;
            break;
          }
          default : {
            response(Gtk::RESPONSE_CANCEL);
          	return;
          }
        }
        break;
      }

      case 7: {
        string message = ("All preparation is now ready.\n" \
                          "Press Apply to actually import the files.");
        AssistantInfoDialog dialog(*this, "Import", ASSISTANT_PAGE_CONFIRM, message);
        int result = dialog.run();
        switch (result) {
          case Gtk::RESPONSE_OK : {
          	// Copy the files to the project.
            string directory = temporary_file(TEMP_DIR);
            ReadFiles rd (directory, "", "");
            for (unsigned int i = 0; i < rd.files.size(); i++) {
              string source = Glib::build_filename(directory, rd.files[i]);
              string destination = projectutils_scripture_filename (myproject, rd.files[i]);
              create_directory(Glib::path_get_dirname(destination));
              string cmd = "cp '";
              cmd.append(source);
              cmd.append("' '");
              cmd.append(destination);
              cmd.append("'");
              Shell shell(cmd);
              shell.run();
            }
          	response(Gtk::RESPONSE_OK);
            return;
          }
          case Gtk::RESPONSE_NO : {
            dialognumber--;
            break;
          }
          default : {
            response(Gtk::RESPONSE_CANCEL);
          	return;
          }
        }
        break;
      }
      
    }
  }
}


string ImportDialog::pick_newest_filename (const vector<string>& filenames) {
/*
 * This function returns the newest filename among the vector of 
 * filenames given.
 */
  // Pick newest.
  unsigned int modificationtime = 0;
  string returnvalue;
  for (unsigned int i = 0; i < filenames.size(); i++) {
    unsigned int modtime = file_get_modification_time(filenames[i]);
    if (modtime > modificationtime) {
      modificationtime = modtime;
      returnvalue = filenames[i];
    }
  }
  return returnvalue;
}

/*
 * Todo
We still need the usfm's from Paratext to connect to the ones 
from Bibledit. Graphical window does this.
Borrow a python script from Paratext that finds 
all standard format markers. 
No, not needed, because we've already code that does that.
Autoconf (AM_SUBST) can fill in the Python path in this file.
We make a script to run, and let it run through Shell.
A sed program then replaces all the markers from one type 
to the other. 
Ensure it replaces only whole words.
*/
