/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogopen.h"
#include "projects.h"
#include "projectutils.h"
#include "directories.h"


OpenDialog::OpenDialog(Gtk::Window& parent, const string& currentproject) :
  Dialog ("Open project", parent, true, false)
{
  // Standard buttons.
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  // Get all projects
  Projects projects (directories_get_projects());
  // Show info.
  if (projects.projects.size() == 0) {
    labelinfo.set_text("There are no projects to open");
  } else {
    labelinfo.set_text("Select the project to open and then press OK");
  }
  labelinfo.set_padding(5, 5);
  get_vbox()->pack_start(labelinfo, false, false, 0);
  // Fill the combo box 
  if (projects.projects.size() > 0) {
    combo.set_popdown_strings(projects.projects);
	if (!currentproject.empty())
      combo.get_entry()->set_text(currentproject);
    combo.get_entry()->set_editable(false);
    get_vbox()->pack_start(combo, false, false, 0);
  } else {
    // No projects, disable OK button.
    set_response_sensitive(Gtk::RESPONSE_OK, false);
  }
  // Show.
  show_all_children();
}


OpenDialog::~OpenDialog() {
}


