/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogprojects.h"
#include "utilities.h"
#include "project.h"
#include "projectutils.h"
#include "projects.h"
#include "directories.h"


ProjectsDialog::ProjectsDialog(Gtk::Window& parent, const string& currentproject) :
  Dialog ("Delete project", parent, true, false)
{
  // Standard buttons.
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
  // Get all projects, and take the one currently opened out.
  vector<string> projectslist;
  Projects projects (directories_get_projects());
  for (unsigned int i = 0; i < projects.projects.size(); i++)
    if (projects.projects[i] != currentproject)
      projectslist.push_back(projects.projects[i]);
  // Show info.
  if (projectslist.size() == 0) {
    labelinfo.set_text("There are no projects to delete");
  } else {
    labelinfo.set_text("Select the project to delete and then press OK");
  }
  labelinfo.set_padding(5, 5);
  get_vbox()->pack_start(labelinfo, false, false, 0);
  if (!currentproject.empty())
    labelmessage.set_text("(the project that is opened is not shown)");
  labelmessage.set_padding(5, 5);
  get_vbox()->pack_start(labelmessage, false, false, 0);
  // Fill the combo box with the projects that can be deleted.
  if (projectslist.size() > 0) {
    combo.set_popdown_strings(projectslist);
    combo.get_entry()->set_text(projectslist[0]);
    combo.get_entry()->set_editable(false);
    get_vbox()->pack_start(combo, false, false, 0);
  } else {
    // No projects, disable OK button.
    set_response_sensitive(Gtk::RESPONSE_OK, false);
  }
  // Show.
  show_all_children();
}


ProjectsDialog::~ProjectsDialog() {
}


void ProjectsDialog::on_response (int response_id) {
  if (response_id == Gtk::RESPONSE_OK) {
    Gtk::MessageDialog dlg1(*this, "Are you sure you want to delete this project?", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
    dlg1.set_default_response(Gtk::RESPONSE_NO);
    int result = dlg1.run();
    if (result == Gtk::RESPONSE_YES) {
      Gtk::MessageDialog dlg2 (*this, "Are you really sure to delete something worth perhaps years of work?", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_YES_NO);
      dlg2.set_default_response(Gtk::RESPONSE_NO);
      int result = dlg2.run();
      if (result == Gtk::RESPONSE_YES) {
        string directory = (Glib::build_filename(directories_get_projects(), combo.get_entry()->get_text()));
	    string command = "rm -rf \"" + Glib::strescape(directory) + "\"";
	    Shell shell(command);
	    shell.run();
      }
    }
  } 
}


