/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogscript.h"
#include "utilities.h"
#include "constants.h"
#include "sys/wait.h"
#include "log.h"


ScriptDialog::ScriptDialog(Gtk::Window& parent, const string& command) :
  Dialog ("Progress...", parent, true, false)
{
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  get_vbox()->pack_start(progressbar, false, false, 10);
  show_all_children();
  log_message ("A script ran with this command:");
  log_message (command);
  log_message ("This is the output the script produced:");
  stream = popen(command.c_str(), "r");
  if (stream) {
    closetimer = 0;
    Glib::signal_timeout().connect(sigc::mem_fun(*this, &ScriptDialog::on_timeout), 20);
    progressbar.set_text("Starting");
  } else {
    progressbar.set_text("Command could not be executed");
  }
}


ScriptDialog::~ScriptDialog() {
  log_message ("End of the script");
}


bool ScriptDialog::on_timeout() {
  // This timer drives the progress bar, etc.
  char buf[1024];
  if (closetimer == 0) {
    if (fgets (buf, sizeof(buf), stream) != NULL) {
      string line = buf;
      line = trim(line);
      log_message (line);
      unsigned int i = line.find("%");
      if (i != string::npos) {
	unsigned int i2 = i - 3;
	if (i2 < 0) i2 = 0;
	progressbar.set_text(line);
	line = number_in_string(line.substr(i2, i2+3)); // Ensure that only the part before the % is checked, and not other numbers.
	progressbar.set_fraction(double(convert_to_int(line)) / 100);
      }
      return true;
    }  else {
      int i = pclose(stream);
      if (i == 0) {
	closetimer++;
	progressbar.set_text("Ready");
	return true;
      } else {
	progressbar.set_text("Error");
	return false;
      }
    }
  } else {
    closetimer++;
    if (closetimer > 20) {      // If ready, hide the window after a short delay.
      hide();			// The delay gives a smoother user interface.
      return false;
    } else {
    }
    return true;
  }
}
