/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogselectbooks.h"
#include "bible.h"


SelectBooksDialog::SelectBooksDialog(Gtk::Window& parent, SelectBookDialogActions action, const vector<string>& selectablebooks, set<string>& selection) :
  Dialog ("Select books", parent, true, false)
{
  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

  Gtk::TreeModel::Row row;

  ref_tree_model1 = Gtk::ListStore::create(columns1);
  treeview1.set_model(ref_tree_model1);
  treeview1.append_column("Law", columns1.col_name);
  unsigned int lastbook = english_id_to_index("Joshua");
  set<string> set1;
  for (unsigned int i = 0; i < lastbook; i++) {
    set1.insert(id_to_biblebook_english(index_to_paratext_id(i)));
  }
  Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection1 = treeview1.get_selection();
  ref_tree_selection1->set_mode(Gtk::SELECTION_MULTIPLE);
  hbox1.pack_start(treeview1, false, false, 0);

  ref_tree_model2 = Gtk::ListStore::create(columns2);
  treeview2.set_model(ref_tree_model2);
  treeview2.append_column("Writings", columns2.col_name);
  unsigned int firstbook = lastbook;
  lastbook = english_id_to_index("Isaiah");
  set<string> set2;
  for (unsigned int i = firstbook; i < lastbook; i++) {
    set2.insert(id_to_biblebook_english(index_to_paratext_id(i)));
  }
  Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection2 = treeview2.get_selection();
  ref_tree_selection2->set_mode(Gtk::SELECTION_MULTIPLE);
  hbox1.pack_start(treeview2, false, false, 0);

  ref_tree_model3 = Gtk::ListStore::create(columns3);
  treeview3.set_model(ref_tree_model3);
  treeview3.append_column("Prophets", columns3.col_name);
  firstbook = lastbook;
  lastbook = english_id_to_index("Matthew");
  set<string> set3;
  for (unsigned int i = firstbook; i < lastbook; i++) {
    set3.insert(id_to_biblebook_english(index_to_paratext_id(i)));
  }
  Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection3 = treeview3.get_selection();
  ref_tree_selection3->set_mode(Gtk::SELECTION_MULTIPLE);
  hbox1.pack_start(treeview3, false, false, 0);

  ref_tree_model4 = Gtk::ListStore::create(columns4);
  treeview4.set_model(ref_tree_model4);
  treeview4.append_column("Gospels, Acts", columns4.col_name);
  firstbook = lastbook;
  lastbook = english_id_to_index("Romans");
  set<string> set4;
  for (unsigned int i = firstbook; i < lastbook; i++) {
    set4.insert(id_to_biblebook_english(index_to_paratext_id(i)));
  }
  Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection4 = treeview4.get_selection();
  ref_tree_selection4->set_mode(Gtk::SELECTION_MULTIPLE);
  hbox1.pack_start(treeview4, false, false, 0);

  ref_tree_model5 = Gtk::ListStore::create(columns5);
  treeview5.set_model(ref_tree_model5);
  treeview5.append_column("Letters", columns5.col_name);
  firstbook = lastbook;
  lastbook = english_id_to_index("James");
  set<string> set5;
  for (unsigned int i = firstbook; i < lastbook; i++) {
    set5.insert(id_to_biblebook_english(index_to_paratext_id(i)));
  }
  Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection5 = treeview5.get_selection();
  ref_tree_selection5->set_mode(Gtk::SELECTION_MULTIPLE);
  hbox1.pack_start(treeview5, false, false, 0);

  ref_tree_model6 = Gtk::ListStore::create(columns6);
  treeview6.set_model(ref_tree_model6);
  treeview6.append_column("Epistles, Rev.", columns6.col_name);
  firstbook = lastbook;
  lastbook = english_id_to_index("Revelation") + 1;
  set<string> set6;
  for (unsigned int i = firstbook; i < lastbook; i++) {
    set6.insert(id_to_biblebook_english(index_to_paratext_id(i)));
  }
  Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection6 = treeview6.get_selection();
  ref_tree_selection6->set_mode(Gtk::SELECTION_MULTIPLE);
  hbox1.pack_start(treeview6, false, false, 0);

  for (unsigned int i = 0; i < selectablebooks.size(); i++) {
    if (set1.find(selectablebooks[i]) != set1.end()) {
      row = *(ref_tree_model1->append());
      row[columns1.col_name] = selectablebooks[i];
    } else if (set2.find(selectablebooks[i]) != set2.end()) {
      row = *(ref_tree_model2->append());
      row[columns2.col_name] = selectablebooks[i];
    } else if (set3.find(selectablebooks[i]) != set3.end()) {
      row = *(ref_tree_model3->append());
      row[columns3.col_name] = selectablebooks[i];
    } else if (set4.find(selectablebooks[i]) != set4.end()) {
      row = *(ref_tree_model4->append());
      row[columns4.col_name] = selectablebooks[i];
    } else if (set5.find(selectablebooks[i]) != set5.end()) {
      row = *(ref_tree_model5->append());
      row[columns5.col_name] = selectablebooks[i];
    } else if (set6.find(selectablebooks[i]) != set6.end()) {
      row = *(ref_tree_model6->append());
      row[columns6.col_name] = selectablebooks[i];
    } 
    myselection = &selection;
  }

  get_vbox()->pack_start(hbox1, false, false, 5);

  string info;
  switch (action) {
    case sbAdd: {
      info = "This is a list of books that can be added to the project";
      if (selectablebooks.size() == 0)
        info.append("\n(none are shown because the project is full)");
      break;
    }
    case sbDelete: {
      info = "This is a list of books that can be deleted from the project";
      if (selectablebooks.size() == 0)
        info.append("\n(none are shown because the project is empty)");
      break;
    }
  }
  infolabel.set_text(info);
  get_vbox()->pack_start(infolabel, false, false, 5);
      
  button_none.set_label("No books");
  button_none.signal_clicked().connect(sigc::mem_fun(*this, &SelectBooksDialog::on_none_clicked));
  hbox2.pack_start(button_none, false, false, 2);

  button_ot.set_label("Old Testament");
  button_ot.signal_clicked().connect(sigc::mem_fun(*this, &SelectBooksDialog::on_ot_clicked));
  hbox2.pack_start(button_ot, false, false, 2);

  button_nt.set_label("New Testament");
  button_nt.signal_clicked().connect(sigc::mem_fun(*this, &SelectBooksDialog::on_nt_clicked));
  hbox2.pack_start(button_nt, false, false, 2);

  button_all.set_label("All books");
  button_all.signal_clicked().connect(sigc::mem_fun(*this, &SelectBooksDialog::on_all_clicked));
  hbox2.pack_start(button_all, false, false, 2);

  get_vbox()->pack_start(hbox2, false, false, 10);

  show_all_children();
}


SelectBooksDialog::~SelectBooksDialog() {
}


void SelectBooksDialog::on_none_clicked() {
  select_row(1, false);  
  select_row(2, false);  
  select_row(3, false);  
  select_row(4, false);  
  select_row(5, false);  
  select_row(6, false);  
}


void SelectBooksDialog::on_ot_clicked() {
  select_row(1, true);  
  select_row(2, true);  
  select_row(3, true);  
  select_row(4, false);  
  select_row(5, false);  
  select_row(6, false);  
}


void SelectBooksDialog::on_nt_clicked() {
  select_row(1, false);  
  select_row(2, false);  
  select_row(3, false);  
  select_row(4, true);  
  select_row(5, true);  
  select_row(6, true);  
}


void SelectBooksDialog::on_all_clicked() {
  select_row(1, true);  
  select_row(2, true);  
  select_row(3, true);  
  select_row(4, true);  
  select_row(5, true);  
  select_row(6, true);  
}


void SelectBooksDialog::on_response (int response_id) {
  if (response_id == Gtk::RESPONSE_OK) {
    // Clear the selection set.
    myselection->clear();
    // Add, through a callback, all selected books.
    {
      Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection = treeview1.get_selection();
      ref_tree_selection->selected_foreach_iter(sigc::mem_fun( *this, &SelectBooksDialog::selected_row_callback) );
    }
    {
      Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection = treeview2.get_selection();
      ref_tree_selection->selected_foreach_iter(sigc::mem_fun(*this, &SelectBooksDialog::selected_row_callback));
    }
    {
      Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection = treeview3.get_selection();
      ref_tree_selection->selected_foreach_iter(sigc::mem_fun(*this, &SelectBooksDialog::selected_row_callback));
    }
    {
      Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection = treeview4.get_selection();
      ref_tree_selection->selected_foreach_iter(sigc::mem_fun(*this, &SelectBooksDialog::selected_row_callback));
    }
    {
      Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection = treeview5.get_selection();
      ref_tree_selection->selected_foreach_iter(sigc::mem_fun(*this, &SelectBooksDialog::selected_row_callback));
    }
    {
      Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection = treeview6.get_selection();
      ref_tree_selection->selected_foreach_iter(sigc::mem_fun(*this, &SelectBooksDialog::selected_row_callback));
    }
  }
}


void SelectBooksDialog::selected_row_callback(const Gtk::TreeModel::iterator& iter) {
  Gtk::TreeModel::Row row = *iter;
  string value;
  row.get_value(0,value);
  myselection->insert(value);
}


void SelectBooksDialog::select_row (int rownumber, bool select) {
  Glib::RefPtr<Gtk::TreeSelection> ref_tree_selection;
  Gtk::TreeModel::iterator iter_begin;
  Gtk::TreeModel::iterator iter_end;
  switch (rownumber) {
  	case 1: { 
  	  ref_tree_selection = treeview1.get_selection();
      iter_begin = ref_tree_model1->children().begin();
      iter_end = ref_tree_model1->children().end();
  	  break;
  	}
  	case 2: { 
  	  ref_tree_selection = treeview2.get_selection();
      iter_begin = ref_tree_model2->children().begin();
      iter_end = ref_tree_model2->children().end();
  	  break;
  	}
  	case 3: { 
  	  ref_tree_selection = treeview3.get_selection();
      iter_begin = ref_tree_model3->children().begin();
      iter_end = ref_tree_model3->children().end();
  	  break;
  	}
  	case 4: { 
  	  ref_tree_selection = treeview4.get_selection();
      iter_begin = ref_tree_model4->children().begin();
      iter_end = ref_tree_model4->children().end();
  	  break;
  	}
  	case 5: { 
  	  ref_tree_selection = treeview5.get_selection();
      iter_begin = ref_tree_model5->children().begin();
      iter_end = ref_tree_model5->children().end();
  	  break;
  	}
  	case 6: { 
  	  ref_tree_selection = treeview6.get_selection();
      iter_begin = ref_tree_model6->children().begin();
      iter_end = ref_tree_model6->children().end();
  	  break;
  	}
  }
  for (Gtk::TreeModel::iterator iter = iter_begin; iter != iter_end; iter++)
    if (select) ref_tree_selection->select(iter);
    else ref_tree_selection->unselect(iter);
}
