/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtkmm.h>
#include "dialogstylesheets.h"
#include "utilities.h"
#include "project.h"
#include "projectutils.h"
#include "projects.h"
#include "stylesheets.h"
#include "stylesheetutils.h"
#include "dialogstylesheet.h"
#include "directories.h"


StylesheetsDialog::StylesheetsDialog(Gtk::Window& parent) :
  Dialog ("Stylesheets", parent, true, false)
{
  // Standard buttons.
  add_button(Gtk::Stock::CLOSE, Gtk::RESPONSE_CLOSE);
  // Combo box with all stylesheets.
  fillcombo("");
  combo.get_entry()->set_editable(false);
  get_vbox()->pack_start(combo, false, false, 0);
  // All buttons.
  newbutton.set_label("New");
  newbutton.signal_clicked().connect(sigc::mem_fun(*this, &StylesheetsDialog::on_new));
  editbutton.set_label("Edit");
  editbutton.signal_clicked().connect(sigc::mem_fun(*this, &StylesheetsDialog::on_edit));
  deletebutton.set_label("Delete");
  deletebutton.signal_clicked().connect(sigc::mem_fun(*this, &StylesheetsDialog::on_delete));
  hbox.pack_start(newbutton, false, false, 0);
  hbox.pack_start(editbutton, false, false, 0);
  hbox.pack_start(deletebutton, false, false, 0);
  get_vbox()->pack_start(hbox, false, false, 0);  
  // Show.
  show_all_children();
}


StylesheetsDialog::~StylesheetsDialog() {
}


void StylesheetsDialog::on_new() {
  stylesheetutils_new("");
  StylesheetDialog dialog(*this, NEW_STYLESHEET);
  if (dialog.run() == Gtk::RESPONSE_OK) {
    fillcombo(dialog.newstylesheetname);
  } 
}


void StylesheetsDialog::on_edit() {
  string stylesheet = combo.get_entry()->get_text();
  StylesheetDialog dialog(*this, stylesheet);
  if (dialog.run() == Gtk::RESPONSE_OK) {
    fillcombo("");
  } 
}


void StylesheetsDialog::on_delete() {
  /*
    Delete the stylesheet, but only when it is not in use by any project.
   */
  // See if the stylesheet is in use.
  string stylesheet = combo.get_entry()->get_text();
  vector<string> projects_using_this_stylesheet = stylesheetutils_lang_used_by(stylesheet);
  if (projects_using_this_stylesheet.size()) {
    // In use: display info about it.
    string message = "This stylesheet could not be deleted, because it is in use by the following project(s):";
    for (unsigned int i = 0; i < projects_using_this_stylesheet.size(); i++)
      message.append("\n- " + projects_using_this_stylesheet[i]);
    Gtk::MessageDialog dialog(*this, message, false, Gtk::MESSAGE_ERROR, Gtk::BUTTONS_OK);
    dialog.run();    
  }
  else {
    // Not in use: delete it.
    string directory = (Glib::build_filename(directories_get_stylesheets(), combo.get_entry()->get_text()));
    string command = "rm -rf \"" + Glib::strescape(directory) + "\"";
    Shell shell(command);
    shell.run();
    fillcombo("");
  }
}


void StylesheetsDialog::fillcombo(const string& active_stylesheet) {
  Stylesheets stylesheets (directories_get_stylesheets());
  if (stylesheets.stylesheets.size() > 0) {
    combo.set_popdown_strings(stylesheets.stylesheets);
    if (active_stylesheet.empty())
      combo.get_entry()->set_text(stylesheets.stylesheets[0]);
    else 
      combo.get_entry()->set_text(active_stylesheet);
  }
}


