/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "constants.h"
#include "utilities.h"
#include "html.h"
#include "bible.h"
#include "usfmtools.h"
#include "book.h"


#define INDEX_HTML "index.html"
#define DOT_HTML ".html"


HtmlConverter::HtmlConverter (int dummy) :
  usfm(0)
{
}


HtmlConverter::~HtmlConverter () {
}


void HtmlConverter::convert (vector<string>& inputfiles, string& outputdirectory) {
  /*
This converts multiple input files to multiple output files,
that is Bibles, in html format
It also creates indexes, for fast navigation between the 
Bible books.
   */


/*

  try {
    cout << "Converting the Bible to directory " << outputdirectory  << " in html format" << endl;
    // Make the output directory if it does not exist.
    create_directory (outputdirectory);
    // Read all information needed for the books, and store it in vectors.
    string input_file;
    string output_file;
    vector<string> filenames;
    vector<string> booknames;
    vector<string> outputfiles;
    for (unsigned int i = 0; i < inputfiles.size(); i++) {
      input_file = inputfiles[i];
      ReadText rt2(input_file);
      Glib::ustring id;
      Glib::ustring s;
      Glib::ustring output;
      Glib::ustring bookname_using_id;
      Glib::ustring bookname_using_running_header;
      Glib::ustring filename;
      output.clear();
      for (unsigned int i = 0; i < rt2.lines.size(); i++) {
	  s = rt2.lines[i];
	  id = usfm_extract_marker (s);
	  if (usfm.is_id (id)) {
	  output = s.substr(0,3);
	  output = id_to_biblebook_english (output);
	  bookname_using_id = output;
	  output.append(DOT_HTML);
	  output = lowerCase(output);
	  output = remove_spaces (output);
	  filename = output;
	}
	if (usfm.is_runningheader (id)) {
	  bookname_using_running_header = s;
	}
      }
      if (bookname_using_running_header.length() > 0) booknames.push_back(bookname_using_running_header);
      else booknames.push_back(bookname_using_id);
      filenames.push_back(filename);
      output_file = outputdirectory;
      output_file.append("/");
      output_file.append(output);
      outputfiles.push_back(output_file);
    }
    // Now convert the whole lot of books.
    for (unsigned int i = 0; i < filenames.size(); i++) {
      string nextoutputfile;
      if (i < (outputfiles.size()-1)) nextoutputfile = outputfiles[i+1];
      string nextbookname;
      if (i < (booknames.size()-1)) nextbookname = booknames[i+1];      
      convert_book (inputfiles[i], outputfiles[i], booknames[i], 100 * i / (filenames.size() - 1), nextoutputfile, nextbookname);
    }    
    // After all files have been converted, use the accumulated information
    // to create an index for all files.
    cout << "Producing index" << endl;
    output_file = outputdirectory;
    output_file.append("/");
    output_file.append(INDEX_HTML);
    WriteText wt(output_file);
    head ("Bible", wt.out);
    wt.out << "<P>";
    for (unsigned int i = 0; i < filenames.size(); i++) {
      wt.out << "<a href=\"" << output_filename (filenames[i], 0, false) << "\">" << booknames[i] << "</a> ";
    }
    wt.out << "</P>" << endl;
    tail(wt.out);
  }
  catch(exception& ex) {
    cerr << "Multiple html converter: " << ex.what() << endl;
  }
  
  
  */
}


void HtmlConverter::head (const string& title, ofstream& out) {
  // Writes the head of an html file, including the title.
  out << "<html>" << endl;
  out << "<head>" << endl;
  out << "<title>" << title << "</title>" << endl;
  out << "</head>" << endl;
  out << "<body>" << endl;
}


void HtmlConverter::tail (ofstream& out) {
  // Writes the tail of an html file, so finishing it.
  out << "</body>" << endl;
  out << "</html>" << endl;
}


void HtmlConverter::convert_chapter (const vector<string>& input, const string& outputfile, const string& bookname, unsigned int currentchapter, const string& nextoutputfile, const string& nextbookname, unsigned int nextchapter) {

  WriteText wt (output_filename (outputfile, currentchapter, false));
  ostringstream r;
  r << bookname << " " << currentchapter;
  head (r.str(), wt.out);
  wt.out << "<p>";
  wt.out << "<a href=\"" << output_filename (outputfile, 0, true) << "\">" << "Chapter index" << "</a>";
  output_next_button (wt.out, nextoutputfile, nextbookname, nextchapter, false);
  wt.out << "</p>" << endl;
  for (unsigned int i = 0; i < input.size(); i++) {
    Glib::ustring line = input[i];
    Glib::ustring usfm = usfm_extract_marker (line);
    output_text (usfm, line, wt.out, false);
  }
  Glib::ustring s;
  output_text (s, s, wt.out, true); // Flush buffer
  output_next_button (wt.out, nextoutputfile, nextbookname, nextchapter, true);
  tail(wt.out);
}


void HtmlConverter::convert_index (const vector<string>& input, const string& outputfile, const string& bookname, unsigned int numberofchapters) {

  WriteText wt (output_filename (outputfile, 0, false));
  head (bookname, wt.out);
  for (unsigned int i = 0; i < input.size(); i++) {
    Glib::ustring line = input[i];
    Glib::ustring usfm = usfm_extract_marker (line);
    output_text (usfm, line, wt.out, false);
  }
  Glib::ustring s;
  output_text (s, s, wt.out, true); // Flush buffers
  wt.out << "<P>";
  wt.out << "<a href=\"" << INDEX_HTML << "\">" << "Bible index" << "</a>";
  wt.out << " - Go to chapter";
  for (unsigned int i = 1; i <= numberofchapters; i++) {
    wt.out << "<a href=\"" << output_filename (outputfile, i, true) << "\"> " << i << " </a>";
  }
  wt.out << "</P>" << endl;
  tail(wt.out);
}


void HtmlConverter::convert_book (const string& inputfile, const string& outputfile, const string& bookname, unsigned int percentage, const string& nextoutputfile, const string& nextbookname) {


/*

  try {
    cout << bookname << " " << percentage << "%" << endl;
    Book book(inputfile, true);
    // Start the conversion.
    vector<string> section;
    bool HeadingDone = false;
    unsigned int CurrentChapter;
    for (unsigned int i = 0; i < book.cleaned_lines.size(); i++) {
      Glib::ustring line = book.cleaned_lines[i];
      Glib::ustring usfm = usfm_extract_marker (line);
      if (usfm.is_chapter(usfm)) {
	if (HeadingDone) {
	  convert_chapter (section, outputfile, bookname, CurrentChapter, outputfile, bookname, CurrentChapter + 1);
	} else {
	  convert_index (section, outputfile, bookname, book.number_of_chapters);
	  HeadingDone = true;
	}
	section.clear();
	CurrentChapter = convert_to_int (number_in_string (line));
      }
      section.push_back(book.cleaned_lines[i]);
    }
    if (section.size()) convert_chapter (section, outputfile, bookname, CurrentChapter, nextoutputfile, nextbookname, 1);
  }
  catch(...) {
    cerr << "Something went wrong in the html converter." << endl;
    throw;
  }
  
  
  */
  
}


string HtmlConverter::output_filename (const string& original_outputfile, unsigned int index, bool trimpath) {
  // Returns original_outputfile with a numeric index added before the suffix.
  // E.g. matthew.html returns matthew000.html
  // If trimpath is true, then only the file part is returned, with the
  // path trimmed off.
  ostringstream r;
  if (index > 99) {
  } else if (index > 9) {
    r << "0";
  } else {
    r << "00";
  }
  r << index;
  string output = original_outputfile;
  output.insert (output.rfind(DOT_HTML), r.str());
  if (trimpath) {
    unsigned int pathpos;
    pathpos = output.rfind("/");
    if (pathpos != string::npos) {
      output.erase(0, ++pathpos);
    }
  }
  return output;
}


void HtmlConverter::output_text (Glib::ustring& usfm, Glib::ustring& text, ofstream& out, bool flushbuffer) {
  /* 
This function outputs any given usfm text into the body
of the html file via pointer out.
  */


/*

  int index = style_sheet.UsfmIndex(usfm);
  // If the usfm is not in the stylesheet, make it verse text.
  // This is a safety measure so that the text is not lost.
  if (index < 0) index = style_sheet.VersetextIndex ();
  if (style_sheet.UsfmNonvernacular (index)) return;
  paragraph (index, text, out, style_sheet.GetUsfmNewParagraph(index) || PreviousWasChapter, flushbuffer);
  // This variabele indicates whether the previous usfm was a chapter. If so,
  // the next lot of text always starts a new paragraph, even if
  // it is character text (which would not normally start a new paragraph).
  // This yields better formatting.
  PreviousWasChapter = usfm.is_chapter(usfm);
  
  
  
  */
  
  
}


void HtmlConverter::paragraph (const unsigned int index, const string& text, ofstream& out, bool startparagraph, bool flush) {



/*

 
  string s;
  if (flush) {
    if (!paragraph_closer.empty()) {
      out << paragraph_closer << endl;
      paragraph_closer.clear();  
    }
    previous_text.clear();
    return;
  }

  // Close the currently open paragraph if need be.
  if (startparagraph) {
    if (!paragraph_closer.empty()) {
      out << paragraph_closer << endl;
      paragraph_closer.clear();  
    }
  }

  if (startparagraph) {
    s = style_sheet.UsfmCenter(index) ? " align=center" : "";
    out << "<p" << s << ">";
    paragraph_closer.insert (0, "</p>");
    out << "<font size=\"" << style_sheet.Fontsize(index) << "\">";
    paragraph_closer.insert (0, "</font>");
    if (style_sheet.UsfmBold(index)) {
      out << "<b>";
      paragraph_closer.insert (0, "</b>");
    }
  }
  if (!startparagraph) {
    if (!previous_text.empty()) {
      out << " ";
    }
  }
  s = text;
  process_text(s);
  out << s;
  previous_text = s;
  // The paragraph is not yet closed now, because there might be more characters
  // that need to be streamed out, but next time this function is called with a 
  // new paragraph starting.
  
  */
  
  
}


void HtmlConverter::add_text (const string& text, string& buffer) {
  // This function adds verse text to the verse buffer,
  // so that it can be written out at once later.
  if (text.empty()) return;
  if (!buffer.empty()) buffer.append(" ");
  buffer.append(text);
}


void HtmlConverter::process_text (string& text) {
  /*
It processes any text, mostly versetext.
Text between square brackets is put in italics.
Example: God [is] our Lord.
The word "is" will be put in italics, and the rest in normal text.
The two square brackets will be removed.
  */


/*

  unsigned int position;
  unsigned int length;
  Glib::ustring usfm;
  int index;
  Glib::ustring endmarker;
  unsigned int endmarkerposition;
  position = text.find("[");
  while (position != string::npos) {
    text.replace(position, 1, "<i>");
    position = text.find("[");
  }
  position = text.find("]");
  while (position != string::npos) {
    text.replace(position, 1, "</i>");
    position = text.find("]");
  }
  position = usfm_search_marker_from (text, 0);
  while (position != string::npos) {
    length = usfm_search_marker_from_length (text, position);
    usfm = text.substr(position, length);
    usfm = usfm_extract_marker (usfm);
    index = style_sheet.UsfmIndex (usfm);
    if (index >= 0) {
      endmarker = style_sheet.GetEndmarker (index);
      if (!endmarker.empty()) {
	if (style_sheet.UsfmBold(index)) {
	  text.replace(position, usfm.length() + 1, "<b>");
	  endmarkerposition = text.find(endmarker, position);
	  if (endmarkerposition != string::npos) {
	    text.replace(--endmarkerposition, endmarker.length() + 1, "</b>");
	  }
	}
	if (style_sheet.UsfmItalics(index)) {
	  text.replace(position, usfm.length() + 1, "<i>");
	  endmarkerposition = text.find(endmarker, position);
	  if (endmarkerposition != string::npos) {
	    text.replace(--endmarkerposition, endmarker.length() + 1, "</i>");
	  }
	}
      }
    }
    position++;
    position = usfm_search_marker_from (text, position);
  }
  
  */
  
  
}


void HtmlConverter::output_next_button (ofstream& out, const string& outputfile, const string& bookname, unsigned int chapter, bool newparagraph) {
  // Write the "Next" link at the end of the chapter.
  if (!bookname.empty()) {
    // Only write this button when there actually is something "next", 
    // so not at the end of the Bible or the end of the set of books.
    if (newparagraph) out << "<p>";
    else out << " - - ";
    out << "<a href=\"" << output_filename (outputfile, chapter, true) << "\">" << "Next";
    if (newparagraph) out << " (" << bookname << " " << chapter << ")";
    else out << " chapter";
    out << "</a>";
    if (newparagraph) out << "</p>" << endl;
  }
}
