/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "mainwindow.h"
#include "libraries.h"
#include <gtkmm.h>
#include "dialogopen.h"
#include "dialogconvert2web.h"
#include "dialogshowscript.h"
#include "constants.h"
#include "config.h"
#include "utilities.h"
#include "htmlbrowser.h"
#include "usfmtools.h"
#include <myconfig.h>
#include "dialogfind.h"
#include "log.h"
#include "dialogcompanion.h"
#include "dialoggotoreference.h"
#include <signal.h>
#include "bible.h"
#include "projectutils.h"
#include "dialogproject.h"
#include "dialogprojects.h"
#include "languageutils.h"
#include "dialoglanguages.h"
#include "stylesheetutils.h"
#include "dialogstylesheets.h"
#include "usfm.h"
#include "dialogimport.h"
#include "worddatabase.h"


/*
|
|
|
|
|
Construction and destruction
|
|
|
|
|
 */


MainWindow::MainWindow() :
  menu_edit_undo("_Undo", Gtk::AccelKey("<ctrl>z"), sigc::mem_fun(*this, &MainWindow::menu_undo)),
  menu_edit_redo("_Redo", Gtk::AccelKey("<ctrl><shift>z"), sigc::mem_fun(*this, &MainWindow::menu_redo)),
  leftarrow(Gtk::ARROW_LEFT, Gtk::SHADOW_ETCHED_IN),
  rightarrow(Gtk::ARROW_RIGHT, Gtk::SHADOW_ETCHED_IN),
  track(0),
  configuration(*this),
  backup(0),
  undo(0),
  windowscompanionlocal(0),
  windowscompanionremote(0)
{
  // Entry in the log.
  log_message("Bibledit starts");

  set_title("Bible Editor");
  try {
    set_icon_from_file(Glib::build_filename(DATAPATH, "icon.xpm"));
  }
  catch (exception) {
  }
  catch (Glib::Error) {
  }

  // Size and position of the main window.
  resize(configuration.window_width, configuration.window_height);
  move (configuration.window_x_position, configuration.window_y_position);

  add(vbox);
  // We pack the menubar into the invisible box, which has been packed into the window.
  // Note that the pack_start default arguments are Gtk::EXPAND | Gtk::FILL, 0
  // The default is not what we want, therefore we enter 0. This causes the menubar not to expand at all,
  // neither to fill. It always stays at the top of the window.
  vbox.pack_start(menubar, 0, 0);
  {
    using namespace Gtk::Menu_Helpers;
    // Deal with the File menu.
    menubar.items().push_back(MenuElem("_File", menufile));
    menufile.items().push_back(StockMenuElem(Gtk::Stock::NEW, sigc::mem_fun(*this, &MainWindow::newproject)));
    menufile.items().push_back(StockMenuElem(Gtk::Stock::OPEN, sigc::mem_fun(*this, &MainWindow::open))); 
    StockMenuElem menufileclose(StockMenuElem(Gtk::Stock::CLOSE, sigc::mem_fun(*this, &MainWindow::close)));
    menufile.items().push_back(menufileclose);
	menu_file_close = menufileclose.get_child();
    menufile.items().push_back(StockMenuElem(Gtk::Stock::DELETE, sigc::mem_fun(*this, &MainWindow::deleteproject)));
    StockMenuElem menufileproperties(StockMenuElem(Gtk::Stock::PROPERTIES, sigc::mem_fun(*this, &MainWindow::editproject)));
    menufile.items().push_back(menufileproperties);
	menu_file_properties = menufileproperties.get_child();
    menufile.items().push_back(MenuElem("Open _references", sigc::mem_fun(*this, &MainWindow::on_open_references)));
    //    menufile.items().push_back(MenuElem("Save references _as", sigc::mem_fun(*this, &MainWindow::on_save_references)));
    MenuElem menufileimport(MenuElem("_Import", sigc::mem_fun(*this, &MainWindow::menu_import)));
    menufile.items().push_back(menufileimport);
	menu_file_import = menufileimport.get_child();
    menufile.items().push_back(StockMenuElem(Gtk::Stock::QUIT, sigc::mem_fun(*this, &MainWindow::hide)));

    // Deal with the Edit menu.
    menubar.items().push_back(MenuElem("_Edit", menuedit));
    menuedit.signal_show().connect( sigc::mem_fun(*this, &MainWindow::menu_edit));
    menuedit.items().push_back(StockMenuElem(Gtk::Stock::CUT, sigc::mem_fun(*this, &MainWindow::on_cut))); 
    menuedit.items().push_back(StockMenuElem(Gtk::Stock::COPY, sigc::mem_fun(*this, &MainWindow::on_copy)));
    menuedit.items().push_back(StockMenuElem(Gtk::Stock::PASTE, sigc::mem_fun(*this, &MainWindow::on_paste)));
    menuedit.items().push_back(menu_edit_undo);
    menuedit.items().push_back(menu_edit_redo);
    menuedit.items().push_back(StockMenuElem(Gtk::Stock::FIND, sigc::mem_fun(*this, &MainWindow::menu_find)));    
    // Deal with the View menu.
    menubar.items().push_back(MenuElem("_View", menuview));
    menuview.items().push_back(StockMenuElem(Gtk::Stock::SELECT_FONT, sigc::mem_fun(*this, &MainWindow::editor_settings)));
    // Deal with the Goto menu.
    menubar.items().push_back(MenuElem("_Goto", menugoto));
    menugoto.items().push_back(MenuElem("Next verse", Gtk::AccelKey("F7"), sigc::mem_fun(*this, &MainWindow::goto_next_verse)));
    menugoto.items().push_back(MenuElem("Previous verse", Gtk::AccelKey("<shift>F7"), sigc::mem_fun(*this, &MainWindow::goto_previous_verse)));
    menugoto.items().push_back(MenuElem("Next chapter", Gtk::AccelKey("F8"), sigc::mem_fun(*this, &MainWindow::goto_next_chapter)));
    menugoto.items().push_back(MenuElem("Previous chapter", Gtk::AccelKey("<shift>F8"), sigc::mem_fun(*this, &MainWindow::goto_previous_chapter)));
    menugoto.items().push_back(MenuElem("Next book", Gtk::AccelKey("F9"), sigc::mem_fun(*this, &MainWindow::goto_next_book)));
    menugoto.items().push_back(MenuElem("Previous book", Gtk::AccelKey("<shift>F9"), sigc::mem_fun(*this, &MainWindow::goto_previous_book)));
    menugoto.items().push_back(MenuElem("Reference", Gtk::AccelKey("<ctrl>G"), sigc::mem_fun(*this, &MainWindow::goto_reference)));
    // Deal with the Tools menu.
    menubar.items().push_back(MenuElem("_Tools", menutools));
    menutools.items().push_back(MenuElem("_Convert project to web pages", sigc::mem_fun(*this, &MainWindow::convert2web)));
    // Deal with the Preferences menu.
    menubar.items().push_back(MenuElem("P_references", menupreferences));
    menupreferences.items().push_back(MenuElem("_Bibledit Windows Companion", sigc::mem_fun(*this, &MainWindow::companion_settings)));
    menupreferences.items().push_back(MenuElem("_Languages", sigc::mem_fun(*this, &MainWindow::language_settings)));
    menupreferences.items().push_back(MenuElem("_Stylesheets", sigc::mem_fun(*this, &MainWindow::stylesheet_settings)));
    // Deal with the Diagnostics menu.
    menubar.items().push_back(MenuElem("_Diagnostics", menudiagnostics));
    menudiagnostics.items().push_back(MenuElem("_View system log", sigc::mem_fun(*this, &MainWindow::viewlog)));
    // Deal with the Help menu.
    menubar.items().push_back(MenuElem("_Help", menuhelp));
    menuhelp.items().push_back(StockMenuElem(Gtk::Stock::HELP, sigc::mem_fun(*this, &MainWindow::showhelp)));
    menuhelp.items().push_back(MenuElem("_About", sigc::mem_fun(*this, &MainWindow::showabout)));
  }

  // Deal with the toolbar.
  vbox.pack_start(hbox, 0, 0);
  backbutton.add(leftarrow);
  backbutton.signal_clicked().connect( sigc::mem_fun(*this, &MainWindow::goto_previous_reference));
  hbox.pack_start(backbutton, 0, 0);
  forwardbutton.add(rightarrow);
  forwardbutton.signal_clicked().connect( sigc::mem_fun(*this, &MainWindow::goto_next_reference));
  hbox.pack_start(forwardbutton, 0, 0);
  hbox.pack_start(combo_book, 0, 0);
  combo_book.get_entry()->signal_changed().connect( sigc::mem_fun(*this, &MainWindow::on_combo_book_changed));
  combo_book.get_entry()->set_editable(false); // Not editable, else the user might enter junk, and crash the program.
  combo_chapter.set_size_request(55);
  hbox.pack_start(combo_chapter, 0, 0);
  combo_chapter.get_entry()->signal_changed().connect( sigc::mem_fun(*this, &MainWindow::on_combo_chapter_changed));
  combo_chapter.get_entry()->set_editable(false);
  combo_verse.set_size_request(55);
  hbox.pack_start(combo_verse, 0, 0);
  combo_verse.get_entry()->signal_changed().connect( sigc::mem_fun(*this, &MainWindow::on_combo_verse_changed));
  combo_verse.get_entry()->set_editable(false);

  // Deal with the editing area.
  hpane.set_position(configuration.hpane_position);
  vbox.pack_start(hpane);

  // Deal with the list area.
  // Add the scrolled window to the right of the moveable pane.
  hpane.add2(scrolledwindow_list);
  // Only show the scrollbars when they are necessary.
  scrolledwindow_list.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
 //Add the treeview inside a scrolledwindow
  scrolledwindow_list.add(treeview);
  //Create the Tree model:
  ref_tree_model = Gtk::ListStore::create(columns);
  treeview.set_model(ref_tree_model);
  //Add the TreeView's view columns:
  treeview.append_column(TEXT_REFERENCES, columns.col_name);
  // Message handler.
  treeview.get_selection()->signal_changed().connect(sigc::mem_fun(*this, &MainWindow::on_list_selection_changed));

  // Deal with the editor area.
  hpane.add1(scrolledwindow_text);
  scrolledwindow_text.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
  scrolledwindow_text.add(textview);
  textview.set_wrap_mode(Gtk::WRAP_WORD);
  textview.signal_grab_focus().connect( sigc::mem_fun(*this, &MainWindow::on_grab_focus));
  textview.get_buffer()->signal_insert().connect( sigc::mem_fun(*this, &MainWindow::on_insert));
  textview.get_buffer()->signal_erase().connect( sigc::mem_fun(*this, &MainWindow::on_erase));
  // Somebody thought that in gtkmm 2.4 the move_cursor signal was useless,
  // so removed it. As we need it here we we trying to go back to the 
  // underlying GTK2 object, and connect the signal from there.
  // on_cursor_moved_tag = g_signal_connect(textview.gobj(), "move_cursor", on_move_cursor, 0);
  // on_cursor_moved_tag = g_signal_connect(textview.gobj(), "move_cursor", *this.on_cursor_moved(), 0);
  // But this does not seem to work.
  // I feel a bit alienated from the gtkmm library because of this,
  // and consider moving to the GTK2 library.
  fonttag = textview.get_buffer()->create_tag("font"); // To set the font properties.
  fonttag->property_font() = configuration.editor_font_name;
  highlighttag = textview.get_buffer()->create_tag("highlight"); // Tag for highlighting text.
  highlighttag->property_background() = "yellow";
  
  // The statusbar.
  statusbar.pack_start(label_current_project);
  vbox.pack_start(statusbar, 0, 0);

  // Show the widgets. They will not really be shown until this window is shown.
  show_all_children();

  // Initalize some variables.
  go_to_reference_running = false;
  scripture = NULL;
  navigator_book = NULL;
  book = NULL;

  // Project initialization.
  initialize_project();

  // Get the system for tracking the Bible references up and running.
  Glib::signal_timeout().connect(sigc::mem_fun(*this, &MainWindow::on_tracker_timeout), 1000);
  set_sensitivity_of_navigator_buttons();

  // System for navigation.
  Glib::signal_timeout().connect(sigc::mem_fun(*this, &MainWindow::on_navigation_timeout), 100);
  Glib::signal_timeout().connect(sigc::mem_fun(*this, &MainWindow::on_cursor_moved), 50);

  // Automatic saving of the file, periodiodically.
  Glib::signal_timeout().connect(sigc::mem_fun(*this, &MainWindow::on_save_timeout), 60000);

  // Communication with Windows Companion
  if (configuration.companion_local_enabled)
    windowscompanionlocal.Connect("localhost", 51515);
  if (configuration.companion_remote_enabled)
    windowscompanionremote.Connect(configuration.companion_remote_name, 51515);
  Glib::signal_timeout().connect(sigc::mem_fun(*this, &MainWindow::on_windows_companion_timeout), 100);

  // Block the signal of a pipe error that otherwise would kill bibledit.
  signal(SIGPIPE, SIG_IGN);

}


MainWindow::~MainWindow() {
  // Save the size and position of the program window.
  // The get_position functions is broken, See the gtkmm documentation for reasons.
  // Anyway, for just now we'll leave the variables in configuration, and we hope
  // that it will be fixed later. The problem now is that the function will
  // return the value that was set with move(x,y). If the user moves the window
  // that is not reflected here, but it will keep returning the values set
  // with move(x,y).
  // If you want to change the default values when Bibledit starts, edit
  // the configuration file
  get_size (configuration.window_width, configuration.window_height);
  get_position(configuration.window_x_position, configuration.window_y_position);
  configuration.hpane_position = hpane.get_position();
  // Save text in editor
  editor_save();
  // Remove system log.
  unlink(LOG_FILE);
  if (scripture) delete scripture;
  if (navigator_book) delete navigator_book;
  if (book) delete book;
  // Optimize word database
  {
    if (!configuration.project.empty()) {
      string command = "bibledit.wdb --finalize ";
      command.append(configuration.project);
      Shell shell (command);
      shell.set_asynchronous();
      shell.run();
    }
  }
}


/*
|
|
|
|
|
Initialization
|
|
|
|
|
 */


void MainWindow::initialize_project() {

  // If we've no project, disables the menus and knock off.
  if (configuration.project.empty()) {
    enable_or_disable_widgets(false);
  	return;
  }

  // Get the Scripture.
  scripture = new Scripture (configuration.project);

  // Update the statusbar.
  label_current_project.set_text(configuration.project);

  // Enable the menus.
  enable_or_disable_widgets(true);

  // Variable needed otherwise the navigation rumbles too much.
  driving_book = true;

  // Put the names of the books in the combo.
  vector<string> strings;
  string gotobook;
  for (unsigned int i = 0; i < scripture->books.size(); i++) {
    strings.push_back(scripture->books[i]);
    if (i == 0) 
      gotobook = scripture->books[i];
  }
  combo_book.set_popdown_strings(strings);
  int bookindex = scripture->get_index_of_book(configuration.book_opened);
  if (bookindex >= 0) {
    // Set the book to the one that was opened last.
    gotobook = scripture->books[bookindex];
  } else {
    // If the book does not exist, reset chapter and verse.
    configuration.chapter_opened = "1";
    configuration.verse_opened = "1";
  }

  // Combo has been filled: clear the anti-rumble variable.
  driving_book = false;

  // Clear some variables so that the new book will be loaded.
  editor_current_book.clear();
  editor_current_chapter.clear();
  editor_current_verse.clear();
  navigator_current_book.clear();
  navigator_current_chapter.clear();
  navigator_current_verse.clear();

  // Go to the new reference.
  if (!gotobook.empty())
    go_to_reference (gotobook, configuration.chapter_opened, configuration.verse_opened, true);

  // Clear the references tracker.
  track.clear();

  // Initialize word database.
  {
    string command = "bibledit.wdb --initialize ";
    command.append(configuration.project);
    Shell shell (command);
    shell.set_asynchronous();
    shell.run();
  }
}


void MainWindow::enable_or_disable_widgets(bool enable) {
  menugoto.set_sensitive(enable);
  menutools.set_sensitive(enable);
  menuedit.set_sensitive(enable);
  menuview.set_sensitive(enable);
  combo_book.set_sensitive(enable);
  combo_chapter.set_sensitive(enable);
  combo_verse.set_sensitive(enable);
  if (!enable) {
    combo_book.get_entry()->set_text("");
    combo_chapter.get_entry()->set_text("");
    combo_verse.get_entry()->set_text("");
  }
  menu_file_properties->set_sensitive(enable);
  menu_file_close->set_sensitive(enable);
  menu_file_import->set_sensitive(enable);
  textview.set_sensitive(enable);
}


/*
|
|
|
|
|
Menu callbacks
|
|
|
|
|
 */


void MainWindow::open() {
  OpenDialog opendialog(*this, configuration.project);
  if (opendialog.run() == Gtk::RESPONSE_OK) {
	close();
    configuration.project = opendialog.combo.get_entry()->get_text();
    initialize_project();
  }
}


void MainWindow::close() {
  editor_save();
  if (book) {
  	delete book;
  	book = NULL;
  }
  enable_or_disable_widgets(false);
  textview.get_buffer()->set_text("");
  textview.get_buffer()->set_modified(false);
  editor_current_book.clear();
  editor_current_chapter.clear();
  editor_current_verse.clear();
  navigator_current_book.clear();
  navigator_current_chapter.clear();
  navigator_current_verse.clear();
  ref_tree_model->clear();
  label_current_project.set_text("");
  configuration.project.clear();
  if (scripture) {
  	delete scripture;
  	scripture = NULL;
  }
  if (navigator_book) {
  	delete navigator_book;
  	navigator_book = NULL;
  }
}


void MainWindow::newproject() {
  projectutils_new();
  ProjectDialog projectdialog(*this, NEW_PROJECT);
  if (projectdialog.run() == Gtk::RESPONSE_OK) {
    close();
    configuration.project = projectdialog.newprojectname;
    initialize_project();
  } 
}


void MainWindow::editproject() {
  ProjectDialog projectdialog(*this, configuration.project);
  if (projectdialog.run() == Gtk::RESPONSE_OK) {
    close();
    configuration.project = projectdialog.newprojectname;
    initialize_project();
  } 
}


void MainWindow::deleteproject() {
  ProjectsDialog dialog(*this, configuration.project);
  dialog.run();
}


void MainWindow::convert2web() {
  ConvertToWebDialog converttoweb(*this, configuration);
  if (converttoweb.run() == Gtk::RESPONSE_OK) {
    configuration.converttowebdirectory = converttoweb.converttowebdirectory;
  }
}


void MainWindow::viewlog() {
  ShowScriptDialog showscript(*this, LOG_FILE);
  showscript.run();
}


void MainWindow::showhelp() {
  htmlbrowser(*this, Glib::build_filename(DATAPATH, "index.html"));

}


void MainWindow::showabout() {
  string message;
  message = "Bibledit ";
  message.append(PACKAGE_VERSION);
  message.append ("\n\n");
  message.append (PACKAGE_BUGREPORT);
  message.append ("\n");
  Gtk::MessageDialog dialog(*this, message);
  dialog.run();
}


void MainWindow::menu_undo() {
  undo.undo(textview.get_buffer());
}


void MainWindow::menu_redo() {
  undo.redo(textview.get_buffer());
}


void MainWindow::menu_edit() {
  // Set the sensitivity of the items under the edit menu.
  menu_edit_undo.get_child()->set_sensitive(undo.can_undo());
  menu_edit_redo.get_child()->set_sensitive(undo.can_redo());
}


void MainWindow:: menu_find() {
  // Before finding, save the current file.
  editor_save();
  // Start find dialog.
  Worddatabase worddatabase(configuration.project);
  FindDialog finddialog(*this, worddatabase.dbfilename());
  finddialog.entry.set_text(configuration.find_searchword);
  if (finddialog.run() == Gtk::RESPONSE_OK) {
    configuration.find_searchword = finddialog.entry.get_text();
    sort_references(finddialog.results);
    fill_list_store(finddialog.results);
  }
}


void MainWindow::companion_settings() {
  CompanionDialog companiondialog (*this, configuration);
  if (companiondialog.run() == Gtk::RESPONSE_OK) {
    // If there is a change in the local connection, deal with it.
    if (configuration.companion_local_enabled != companiondialog.checkbuttonlocal.get_active()) {
      configuration.companion_local_enabled = companiondialog.checkbuttonlocal.get_active();
      if (configuration.companion_local_enabled) {
	windowscompanionlocal.Connect("localhost", 51515);
      } else {
	windowscompanionlocal.Disconnect();
      }
    }
    // Deal with any changes in the remote connection.
    bool b = false;
    if (configuration.companion_remote_enabled != companiondialog.checkbuttonremote.get_active()) 
      b = true;
    if (configuration.companion_remote_name.compare(companiondialog.entry.get_text()))
      b = true;
    if (b) {
      configuration.companion_remote_enabled = companiondialog.checkbuttonremote.get_active();
      configuration.companion_remote_name = companiondialog.entry.get_text();
      if (configuration.companion_remote_enabled) {
	windowscompanionremote.Connect(configuration.companion_remote_name, 51515);
      } else {
	windowscompanionremote.Disconnect();
      }
    }
  }
}


void MainWindow::editor_settings() {
  Gtk::FontSelectionDialog dialog;
  dialog.set_font_name(configuration.editor_font_name);
  int result = dialog.run();
  if (result == Gtk::RESPONSE_OK) {
    configuration.editor_font_name = dialog.get_font_name();
    fonttag->property_font() = configuration.editor_font_name;
  }
}


void MainWindow::language_settings() {
  LanguagesDialog dialog(*this);
  dialog.run();
}


void MainWindow::stylesheet_settings() {
  StylesheetsDialog dialog(*this);
  dialog.run();
}


void MainWindow::menu_import() {
  ImportDialog dialog(*this, configuration.project);
  int result = dialog.run();
  if (result == Gtk::RESPONSE_OK) {
    string prj = configuration.project;
	close();
	configuration.project = prj;
    initialize_project();
  }
}


/*
|
|
|
|
|
Navigation
|
|
|
|
|
 */


void MainWindow::on_combo_book_changed() {
  if (driving_book) return;
  string book = combo_book.get_entry()->get_text();
  if (book.empty()) return;
  go_to_reference(book, "", "", false);
}


void MainWindow::on_combo_chapter_changed() {
  if (driving_chapter) return;
  string chapter = combo_chapter.get_entry()->get_text();
  if (chapter.empty()) return;
  go_to_reference("", chapter, "", false);
}


void MainWindow::on_combo_verse_changed() {
  if (driving_verse) return;
  string verse = combo_verse.get_entry()->get_text();
  if (verse.empty()) return;
  go_to_reference("", "", verse, false);
   
}


void MainWindow::goto_next_verse() {
  set_next_or_previous_item(combo_verse, combo_verse.get_entry()->get_text(), true);
}


void MainWindow::goto_previous_verse() {
  set_next_or_previous_item(combo_verse, combo_verse.get_entry()->get_text(), false);
}


void MainWindow::goto_next_chapter() {
  set_next_or_previous_item(combo_chapter, combo_chapter.get_entry()->get_text(), true);
}


void MainWindow::goto_previous_chapter() {
  set_next_or_previous_item(combo_chapter, combo_chapter.get_entry()->get_text(), false);
}


void MainWindow::goto_next_book() {
  set_next_or_previous_item(combo_book, combo_book.get_entry()->get_text(), true);
}


void MainWindow::goto_previous_book() {
  set_next_or_previous_item(combo_book, combo_book.get_entry()->get_text(), false);
}


void MainWindow::goto_reference() {
  GotoReferenceDialog dialog(*this, navigator_current_book, navigator_current_chapter, navigator_current_verse);
  if (dialog.run() == Gtk::RESPONSE_OK) {
    if (dialog.newreference)
      go_to_reference (dialog.newbook, dialog.newchapter, dialog.newverse, true);
  }
}


void MainWindow::go_to_reference(const string& gotobook, const string& gotochapter, const string& gotoverse, bool jump) {
  /*
    This function makes the editor to go to a new Bible reference.

    Going to a reference is not done immediately on request. It is instead done by a scheduler.
    This is done because at times the program generates a lot of requests at once.
    If the edit control is then loaded each time, and the cursor positioned,
    this confuses Gtk when the requests come too soon one after the other.
    So this function only stores the requested references, and a timer will carry
    this request out later. 

    If the previously displayed reference was edited and therefore changed,
    it will save the file.

    The "jump" variabele needs to be set when any arbitrary jump is made.
    It is related to some internal logic.
   */


  // If we're busy bail out.
  if (go_to_reference_running) return;
  go_to_reference_running = true;

  try {

  // Decide where we're going to.
  string navigator_goto_book;
  if (gotobook.empty())
    navigator_goto_book = navigator_current_book;
  else
    navigator_goto_book = gotobook;
  string navigator_goto_chapter;
  if (gotochapter.empty())
    navigator_goto_chapter = navigator_current_chapter;
  else
    navigator_goto_chapter = gotochapter;
  string navigator_goto_verse;
  if (gotoverse.empty())
    navigator_goto_verse = navigator_current_verse;
  else
    navigator_goto_verse = gotoverse;

  // Find out what needs to be changed: book, chapter and/or verse.
  bool new_book = (navigator_goto_book != navigator_current_book);
  bool new_chapter = (navigator_goto_chapter != navigator_current_chapter);
  bool new_verse = (navigator_goto_verse != navigator_current_verse);

  // Index of the book in the currently opened scripture.
  int bookindex = -1;
  if (scripture) 
    bookindex = scripture->get_index_of_book(navigator_goto_book);

  // See whether the requested book exists.
  if (bookindex < 0) {
    // It doesn't exist, so do nothing.
    new_book = false;
    new_chapter = false;
    new_verse = false;
  }

  // Deal with a new book.
  if (new_book) {
    // Adjust the current indicator, and store the book for later.
    navigator_goto_book = scripture->books[bookindex];
    navigator_current_book = navigator_goto_book;
    configuration.book_opened = navigator_goto_book;
    // Set the text in the combo box.
    driving_book = true;
    combo_book.get_entry()->set_text(navigator_goto_book);
    driving_book = false;
    // Load the book in memory. It'll provide data on the number of chapters and verses.
    if (navigator_book)
      delete navigator_book;
    navigator_book = new Book (scripture->paths[bookindex], false);
    // Put the chapters in the chapter combo box.
    vector<string> strings;
    for (unsigned int i = 1; i <= navigator_book->number_of_chapters; i++) {
      strings.push_back(convert_to_string(i));
    }
    driving_chapter = true;
    combo_chapter.set_popdown_strings(strings);
    driving_chapter = false;
    // As we've a new book, go to chapter one verse one, except when "jumping".
    if (!jump) {
      navigator_goto_chapter = "1";
      navigator_goto_verse = "1";
    }
    new_chapter = true;
  }

  // Bring the new chapter within the limits
  if (new_chapter) {
    unsigned int int_goto_chapter = convert_to_int(navigator_goto_chapter);
    if (int_goto_chapter < 1)
      navigator_goto_chapter = "1";
    if (int_goto_chapter > navigator_book->number_of_chapters)
      navigator_goto_chapter = convert_to_string(navigator_book->number_of_chapters);
  }

  // Deal with a new chapter
  if (new_chapter) {
    // Set the text in the combo box.
    driving_chapter = true;
    combo_chapter.get_entry()->set_text(navigator_goto_chapter);
    driving_chapter = false;
    // Store the current chapter.
    navigator_current_chapter = navigator_goto_chapter;
    configuration.chapter_opened = navigator_current_chapter;
    // Put the number of verses in combo_verse
    vector<string> strings;
    unsigned int chapter = convert_to_int (navigator_goto_chapter);
    if (chapter > 0) {
      unsigned int number_of_verses = navigator_book->number_of_verses(chapter);
      for (unsigned int i = 1; i <= number_of_verses; i++) {
	strings.push_back(convert_to_string(i));
      }
      driving_verse = true;
      combo_verse.set_popdown_strings(strings);
      driving_verse = false;
    }
    // Move to beginning of the chapter, except when "jumping"
    if (!jump)
      navigator_goto_verse = "1";
    new_verse = true;
  }

  // Bring the new verse within its limits.
  if (new_verse) {
    int int_goto_verse = convert_to_int(navigator_goto_verse);
    int limit = navigator_book->number_of_verses(convert_to_int(navigator_goto_chapter));
    if (int_goto_verse < 1) navigator_goto_verse = "1";
    if (int_goto_verse > limit) navigator_goto_verse = convert_to_string(limit);
  }

  // Deal with a new verse.
  if (new_verse) {
    // Set the text in the combo box.
    driving_verse = true;
    combo_verse.get_entry()->set_text(navigator_goto_verse);
    driving_verse = false;
    // Update the current verse, and store it for later also.
    navigator_current_verse = navigator_goto_verse;
    configuration.verse_opened = navigator_current_verse;
  }

  // If we've changed a reference: reset the timer,
  // allowing the navigation timeout to update the editor a bit later.
  if (new_book || new_chapter || new_verse) 
    navigation_change_timer = 0;

  }
  catch (exception) {
  }
  catch (Glib::Error) {
  }
  
  // Ready running.
  go_to_reference_running = false;
}


bool MainWindow::on_navigation_timeout() {
  /*
    This scheduler carries out requested changes of the reference.
    It also does related tasks.
  */

  // After a short while remove the highlight that has previously been put on a new verse.
  remove_highlight_timer++;
  if (remove_highlight_timer == 20)
    textview.get_buffer()->remove_tag (highlighttag, textview.get_buffer()->begin(), textview.get_buffer()->end());

  // If F7, F8 or F9 is continually held down and it repeats,
  // we've code here to deal with that so that the interface responds
  // properly. In cases like this the change of the reference
  // is delayed until the key is released.
  navigation_change_timer++;
  if (navigation_change_timer != 4)
    return true;

  // Find out what needs to be changed: book, chapter and/or verse.
  string goto_book = combo_book.get_entry()->get_text();
  bool new_book = (goto_book != editor_current_book);
  editor_current_book = goto_book;
  string goto_chapter = combo_chapter.get_entry()->get_text();
  bool new_chapter = (goto_chapter != editor_current_chapter);
  editor_current_chapter = goto_chapter;
  string goto_verse = combo_verse.get_entry()->get_text();
  bool new_verse = (goto_verse != editor_current_verse);
  editor_current_verse = goto_verse;

  // See whether the current file should be saved.
  if (new_book || new_chapter) {
    if (textview.get_buffer()->get_modified()) {
      editor_save();
    }
  }

  // Deal with a new book.
  if (new_book) {
    int bookindex = scripture->get_index_of_book(goto_book);
    // Load book, do not split the lines.
    if (book) delete book;
    book = new Book (*this, scripture->paths[bookindex], false);
    // With a new book, also load a new chapter.
    new_chapter = true;
  }

  // Deal with a new chapter.
  if (new_chapter) {
    // Load chapter, if need be.
    editor_load_chapter();
    // When loading a new chapter, we've also a new verse.
    new_verse = true;
  }

  if (new_book || new_chapter || new_verse) {
    // Position the cursor properly.
    // The positioning will be done whenever Gtk is idle.
    // This is because sometimes Gtk is slow in loading a new chapter.
    // So if the cursor positioning is done straight after loading,
    // it might not work.
    goto_cursor_position = goto_verse;
    Glib::signal_idle().connect(sigc::mem_fun(*this, &MainWindow::editor_position_cursor_at_verse));
    // Send the reference to the windows companions.
    string value = "gotoreference " + goto_book.substr(0, 3) + " " + goto_chapter + ":" + goto_verse;
    windowscompanionlocal.Sendln(value);
    windowscompanionremote.Sendln(value);
  }

  return true;
}


bool MainWindow::on_cursor_moved() {
  /*
    When the cursor has moved, the navigation system needs to be updated
    so that it shows the right reference. If the user was, for example
    on MAT 1:10, and the cursor moves, the move might have brought him or her
    to another reference, though this is not necessarily so. Therefore, as we 
    don't know where the user is now after the cursor moved, we need to find
    it out. The book is known, the chapter is known, because both stay the same.
    The only thing we don't know is the verse. 
   */

/*
  gtkmm 2.4 removed signal_cursor_moved, so bibledit is now polling the
  line the cursor is on, and takes action when it has changed.
  */

  static int previous_cursor_line = 0;
  /* Initializing the value to 0 instead of -1 solves a Segmentation fault 
   * exception when no document was loaded.
   */
  Gtk::TextIter iterator (textview.get_buffer()->get_iter_at_mark(textview.get_buffer()->get_insert()));
  int line_of_cursor = iterator.get_line();
  if (line_of_cursor != previous_cursor_line) {
  	previous_cursor_line = line_of_cursor;
    go_to_reference ("", "", cursor_get_verse_number(), false);
  }
  return true;
}


void MainWindow::on_grab_focus() {
  // The purpose of this is to adjust the reference to the new position,
  // wherever the user clicks in the text.
  if (focus_programmatically_being_grabbed) return;
  Glib::signal_idle().connect(sigc::mem_fun(*this, &MainWindow::on_grab_focus_delayed_handler));
}


string MainWindow::cursor_get_verse_number() {
  /*
    This gets the verse number the cursor is on.
    We will find the verse out by doing the following things:
    1. Get the line the cursor is on. If it has a verse marker, we then can get
       the verse number from there.
    2. If we did not find a verse number, get the previous line, and see what
       information is there.
    3. Keep doing step 2 until we either have the information we want, or we have
       arrived at the beginning of the file.
    4. If we are at the start of the file, assume verse one.
   */
  // Get the line the cursor is on.
  Gtk::TextIter iterator (textview.get_buffer()->get_iter_at_mark(textview.get_buffer()->get_insert()));
  int line_of_cursor = iterator.get_line();
  // If no verse information is going to be found, assume verse one.
  string verse = "1";
  // Go through all lines, from the cursor downwards, and check whether verse information is there.
  Gtk::TextIter endofline;
  Usfm usfm(0);
  for (int i = line_of_cursor; i >= 0; i--) {
    iterator = textview.get_buffer()->get_iter_at_line(i);
    endofline = (textview.get_buffer()->get_iter_at_line(i+1));
    endofline--;
    Glib::ustring s = textview.get_buffer()->get_text(iterator, endofline);
    Glib::ustring marker = usfm_extract_marker (s);
    if (usfm.is_verse(marker)) {
      verse = number_in_string (s);
      break;
    }
  }
  return verse;
}


bool MainWindow::editor_position_cursor_at_verse() {
  /*
    This function moves the cursor of the editor to the verse stored before.
  */

  // We will not move the cursor or scroll to it when the cursor is already on the right verse.
  bool b = (goto_cursor_position.compare (cursor_get_verse_number()));
  // When the cursor is at the end of the buffer, it usually means that the buffer has just been filled.
  // So the cursor does need to be repositioned in this case.
  Gtk::TextIter cursor_iterator (textview.get_buffer()->get_iter_at_mark(textview.get_buffer()->get_insert()));
  if (cursor_iterator == textview.get_buffer()->end()) b = true;
  if (b) {
    // Grab focus here to get the scrolling done properly, and the user can type in it.
    focus_programmatically_being_grabbed = true;
    textview.grab_focus();
    focus_programmatically_being_grabbed = false;

	// Go through all the lines and find out about the verse.
    Usfm usfm(0);
	for (int i = 0; i < textview.get_buffer()->get_line_count(); i++) {
      Gtk::TextIter begin = textview.get_buffer()->get_iter_at_line(i);
      Gtk::TextIter end = textview.get_buffer()->get_iter_at_line(i+1);
      Glib::ustring line = textview.get_buffer()->get_text(begin, end);
      Glib::ustring marker = usfm_extract_marker (line);
      if (usfm.is_verse(marker)) {
        Glib::ustring verse = number_in_string (line);
        if (verse == goto_cursor_position) {
          // Move the cursor to it.
          // Move it to the beginning of the text, if there is any.
          int offset = 3 + marker.length() + verse.length();
          int linelength = textview.get_buffer()->get_text(begin, end).length();
          linelength--;
          if (offset >= linelength) offset = linelength;
          begin = textview.get_buffer()->get_iter_at_line_offset(i, offset);
          textview.get_buffer()->place_cursor(begin);
          // Scroll also to it. It will scroll to the beginning of the text after the verse marker.
          // Alignment is needed to put the line being edited near the top of the window.
          textview.scroll_to(begin, 0.1); 
          // Highlight the verse we moved to (remove any previous tags)
          textview.get_buffer()->remove_tag (highlighttag, textview.get_buffer()->begin(), textview.get_buffer()->end());
          int linenumber = begin.get_line();
          textview.get_buffer()->apply_tag (highlighttag, textview.get_buffer()->get_iter_at_line(linenumber), textview.get_buffer()->get_iter_at_line(linenumber++));
          remove_highlight_timer = 0;
          break;
        }
      }
	}
  }
  return false;
}


/*
|
|
|
|
|
Tracker
|
|
|
|
|
 */


void MainWindow::goto_next_reference() {
  set_next_or_previous_reference(true);
}


void MainWindow::goto_previous_reference() {
  set_next_or_previous_reference(false);
}


bool MainWindow::set_next_or_previous_item(Gtk::Combo& combo, const string& current_text, bool next) {
  // This function takes the next (or previous) value in the dropdown list of the combo box.
  // It returns true if a next (or previous) value could be set, else false.
  vector<string> strings;
  strings = combo.get_popdown_strings();
  if (strings.size() == 0) return false;
  unsigned int index = 0;
  for (unsigned int i = 0; i < strings.size(); i++) {
    if (current_text == strings[i])
      index = i;
  }
  if (next) {
    if (index == (strings.size() - 1)) return false;
    index++;
  } else {
    if (index == 0) return false;
    index--;
  }
  string combo_text;
  combo_text = strings[index];
  // Go to the new reference.
  if (&combo == &combo_book) go_to_reference (combo_text, "", "", false);
  if (&combo == &combo_chapter) go_to_reference ("", combo_text, "", false);
  if (&combo == &combo_verse) go_to_reference ("", "", combo_text, false);
  // Return true as we've done the job.
  return true;
}


void MainWindow::set_sensitivity_of_navigator_buttons() {
  // Enables or disables each navigator button, depending on whether references for it are available.
  backbutton.set_sensitive(track.previous_reference_available());
  forwardbutton.set_sensitive(track.next_reference_available());
}


bool MainWindow::on_tracker_timeout() {
  // Drive the Bible reference tracker.
  track.tick(combo_book.get_entry()->get_text(), combo_chapter.get_entry()->get_text(), combo_verse.get_entry()->get_text());
  set_sensitivity_of_navigator_buttons();
  return true;
}


void MainWindow::set_next_or_previous_reference(bool next) {
  string book;
  string chapter;
  string verse;
  track.get_next_reference (book, chapter, verse, next);
  go_to_reference (book, chapter, verse, true);
  set_sensitivity_of_navigator_buttons();
}


bool MainWindow::on_grab_focus_delayed_handler() {
  /*
    If the user clicks in the editor window, and straight after that
    the verse where the cursor is, is asked, we get a reply 
    indicating the verse where the cursor was prior to clicking.
    This delayed handler solves that.
  */
  go_to_reference ("", "", cursor_get_verse_number(), false);
  return false;
}


/*
|
|
|
|
|
Editing
|
|
|
|
|
 */


void MainWindow::editor_load_chapter() {
  /*
    Loads the chapter in the editor.
  */
  // Set the new text.
  vector<Glib::ustring> lines;
  book->get_chapter(convert_to_int(combo_chapter.get_entry()->get_text()), lines);
  Glib::ustring text;
  for (unsigned int i = 0; i < lines.size(); i++) {
    text.append(lines[i] + "\n");
  }
  textview.get_buffer()->set_text(text);
  // Apply the right font.
  textview.get_buffer()->apply_tag (fonttag, textview.get_buffer()->begin(), textview.get_buffer()->end());
  // Set the editor ummodified.
  textview.get_buffer()->set_modified(false);
  // Clear the undo / redo stack.
  undo.clear();
}


void MainWindow::editor_save() {
  /*
    This will save the book if it has been changed in the editor.
   */
  // See whether it was changed.
  if (textview.get_buffer()->get_modified()) {
    // Collect all lines of the editor
    vector<Glib::ustring> lines;
    int number_of_lines = textview.get_buffer()->get_line_count();
    Gtk::TextIter iterator;
    Gtk::TextIter endofline;
    for (int i = 0; i < number_of_lines; i++) {
      iterator = textview.get_buffer()->get_iter_at_line(i);
      endofline = (textview.get_buffer()->get_iter_at_line(i+1));
      endofline--;
      string s = textview.get_buffer()->get_text(iterator, endofline);
      s = trim(s);
      if (!s.empty()) lines.push_back(s);
    }
    // Rejoin the lines in the chapter and save the whole book.
    book->set_chapter(lines);
    book->save();
    // Set the buffer to unmodified.
    textview.get_buffer()->set_modified(false);
    // Start the process that will update the word database.
    string command = "bibledit.wdb --update ";
    command.append(configuration.project);
    command.append(" '");
    command.append(index_to_biblebook(book->get_id_index()));
    command.append("' ");
    command.append(convert_to_string((int)book->get_previously_presented_chapter()));
    Shell shell (command);
    shell.set_asynchronous();
    shell.run();
  }
}


bool MainWindow::on_save_timeout() {
  editor_save();
  return true;
}


/*
|
|
|
|
|
Clipboard
|
|
|
|
|
 */


void MainWindow::on_cut() {
  Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
  if (textview.has_focus())
    textview.get_buffer()->cut_clipboard(refClipboard);
}


void MainWindow::on_copy() {
  Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
  if (textview.has_focus())
    textview.get_buffer()->copy_clipboard(refClipboard);
}


void MainWindow::on_paste() {
  Glib::RefPtr<Gtk::Clipboard> refClipboard = Gtk::Clipboard::get();
  if (textview.has_focus())
    textview.get_buffer()->paste_clipboard(refClipboard);
}


/*
|
|
|
|
|
Backups
|
|
|
|
|
 */


/*
|
|
|
|
|
List store and reference handling
|
|
|
|
|
 */


void MainWindow::fill_list_store(vector<string>& lines) {
  /*
    Fill the list store with the data.
   */
  // Heading in column
  treeview.remove_all_columns();
  string s = TEXT_REFERENCES;
  s.append (" - ");
  s.append (convert_to_string(int(lines.size())));
  treeview.append_column(s, columns.col_name);
  // Clear the store first.
  ref_tree_model->clear();
  // Now add new data.
  Gtk::TreeModel::Row row;
  for (unsigned int i = 0; i < lines.size(); i++) {
    row = *(ref_tree_model->append());
    row[columns.col_name] = lines[i];
  }
}


void MainWindow::on_list_selection_changed() {
  /*
    Handler if the selection in the list changed
  */
  try {

    // Get the string the user selected.
    Gtk::TreeModel::iterator iter = treeview.get_selection()->get_selected();
    if(iter) {
      Gtk::TreeRow row = *iter;
      string value;
      row.get_value(0,value);
      // Extract the book.
      int booklength;
      booklength = value.find(" ", 2);
      string book;
      book = value.substr(0, booklength);
      value.erase(0, booklength);
      value = trim(value);
      // Extract chapter.
      string chapter;
      chapter = number_in_string(value);
      value.erase(0, chapter.length());
      value = trim(value);
      // Extract verse.
      string verse;
      verse = number_in_string(value);
      // Jump to the reference.
      go_to_reference(book, chapter, verse, true);
    }
  }
  catch (exception &ex) {
  	cout << ex.what() << endl;
  }
  catch (Glib::Error &err) {
    cout << err.what() << endl;
  }
}


void MainWindow::on_open_references() {
  Gtk::FileSelection dialog("Please choose a file");
  dialog.set_transient_for(*this);
  dialog.set_filename(configuration.references_file);
  int result = dialog.run();
  if (result == Gtk::RESPONSE_OK) { 
    configuration.references_file = dialog.get_filename();
    ReadText rt(configuration.references_file, true);
    sort_references(rt.lines);
    fill_list_store(rt.lines);
  }
}


void MainWindow::on_save_references() {

}


/*
|
|
|
|
|
Undo / Redo
|
|
|
|
|
 */


void MainWindow::on_insert(const Gtk::TextBuffer::iterator& pos, const Glib::ustring& text, int bytes) {
  undo.on_insert(pos, text, bytes);
}


void MainWindow::on_erase(const Gtk::TextBuffer::iterator& range_begin, const Gtk::TextBuffer::iterator& range_end) {
  undo.on_erase(range_begin, range_end);
}


/*
|
|
|
|
|
Bibledit Windows Companion
|
|
|
|
|
 */


bool MainWindow::on_windows_companion_timeout() {
  if (configuration.companion_local_enabled)
    windowscompanionlocal.Tick();
  if (configuration.companion_remote_enabled)
    windowscompanionremote.Tick();
  return true;
}


// Todo Let the references be displayed using English names.
