/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef INCLUDED_MAINWINDOW_H
#define INCLUDED_MAINWINDOW_H

#include <gtkmm.h>
#include "configuration.h"
#include "track.h"
#include "book.h"
#include "backup.h"
#include "undo.h"
#include "windowscompanion.h"
#include "scripture.h"


class MainWindow : public Gtk::Window
{
public:
  MainWindow();
  virtual ~MainWindow();
protected:
  Gtk::VBox vbox;
  Gtk::MenuBar menubar;
  /* IMPORTANT If more menus are added see whether they should be made in(sensitive) depending on whether a project has been opened. */
  Gtk::Menu menufile;
  Glib::RefPtr<Gtk::MenuItem> menu_file_properties;
  Glib::RefPtr<Gtk::MenuItem> menu_file_close;
  Glib::RefPtr<Gtk::MenuItem> menu_file_import;
  Gtk::Menu menuedit;
  Gtk::Menu_Helpers::MenuElem menu_edit_undo;
  Gtk::Menu_Helpers::MenuElem menu_edit_redo;
  Gtk::Menu menuview;
  Gtk::Menu menugoto;
  Gtk::Menu menutools;
  Gtk::Menu menupreferences;
  Gtk::Menu menudiagnostics;
  Gtk::Menu menuhelp;
  Gtk::HBox hbox;
  Gtk::Button backbutton;
  Gtk::Arrow leftarrow;
  Gtk::Button forwardbutton;
  Gtk::Arrow rightarrow;
  Gtk::Combo combo_book;
  Gtk::Combo combo_chapter;
  Gtk::Combo combo_verse;
  Gtk::HPaned hpane;
  Gtk::ScrolledWindow scrolledwindow_text;
  Gtk::TextView textview;
  Glib::RefPtr<Gtk::TextTag> fonttag;
  Glib::RefPtr<Gtk::TextTag> highlighttag;
  Gtk::ScrolledWindow scrolledwindow_list;
  Gtk::TreeView treeview;
  class ModelColumns : public Gtk::TreeModel::ColumnRecord {
  public:
    ModelColumns()
    { add(col_name); }
    Gtk::TreeModelColumn<string> col_name;
  };
  ModelColumns columns;
  Glib::RefPtr<Gtk::ListStore> ref_tree_model;
  Gtk::Statusbar statusbar;
  Gtk::Label label_current_project;

  /* Initialization */
  virtual void initialize_project();
  virtual void enable_or_disable_widgets(bool enable);

  /* Menu callbacks */
  virtual void open();
  virtual void close();
  virtual void newproject();
  virtual void editproject();
  virtual void deleteproject();
  virtual void convert2web();
  virtual void viewlog();
  virtual void showhelp();
  virtual void showabout();
  virtual void menu_undo();
  virtual void menu_redo();
  virtual void menu_edit();
  virtual void menu_find();
  virtual void companion_settings();
  virtual void editor_settings();
  virtual void language_settings();
  virtual void stylesheet_settings();
  virtual void menu_import();

  /* Navigation  */
  virtual void on_combo_book_changed();
  virtual void on_combo_chapter_changed();
  virtual void on_combo_verse_changed();
  virtual void goto_next_verse();
  virtual void goto_previous_verse();
  virtual void goto_next_chapter();
  virtual void goto_previous_chapter();
  virtual void goto_next_book();
  virtual void goto_previous_book();
  virtual void goto_reference();
  virtual void go_to_reference(const string& gotobook, const string& gotochapter, const string& gotoverse, bool jump);
  bool go_to_reference_running;
  string editor_current_book, editor_current_chapter, editor_current_verse;
  virtual bool on_navigation_timeout();
  bool driving_book, driving_chapter, driving_verse;
  virtual bool on_cursor_moved();
  bool focus_programmatically_being_grabbed;
  virtual void on_grab_focus();
  string cursor_get_verse_number();
  unsigned int on_grab_focus_timer;
  string goto_cursor_position;
  unsigned int navigation_change_timer;
  string navigator_current_book, navigator_current_chapter, navigator_current_verse;
  bool on_grab_focus_delayed_handler();
  unsigned int remove_highlight_timer;
  
  /* Tracker */
  virtual void goto_next_reference();
  virtual void goto_previous_reference();
  virtual bool set_next_or_previous_item(Gtk::Combo& combo, const string& current_text, bool next);
  virtual void set_sensitivity_of_navigator_buttons();
  virtual bool on_tracker_timeout();
  virtual void set_next_or_previous_reference(bool next);
  Track track;

  /* Variables for general use */
  Configuration configuration;
  Scripture* scripture;
  Book* navigator_book;  

  /* Loading, editing, saving */
  Book* book;
  virtual void editor_load_chapter();
  virtual bool editor_position_cursor_at_verse();
  virtual void editor_save();
  virtual bool on_save_timeout();

  /* Clipboard */
  virtual void on_cut();
  virtual void on_copy();
  virtual void on_paste();

  /* Backups */
  Backup backup;

  /* List store and reference handling */
  virtual void fill_list_store(vector<string>& lines);
  virtual void on_list_selection_changed();
  virtual void on_open_references();
  virtual void on_save_references();

  /* Undo and Redo */
  Undo undo;
  virtual void on_insert(const Gtk::TextBuffer::iterator& pos, const Glib::ustring& text, int bytes);
  virtual void on_erase(const Gtk::TextBuffer::iterator& range_begin, const Gtk::TextBuffer::iterator& range_end);

  /* Undo and Redo */
  virtual bool on_windows_companion_timeout();

  /* Bibledit Windows Companion */
  WindowsCompanion windowscompanionlocal;
  WindowsCompanion windowscompanionremote;

};


#endif


