/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "track.h"


Track::Track (unsigned int dummy) {
  clear();
}


Track::~Track () {
}


void Track::tick(const string& book, const string& chapter, const string& verse) {
  string current_reference = reference (book, chapter, verse);
  // The recorded reference variable contains the reference that has been recorded.
  // This is usually the last reference that was recorded,
  // except in cases that the forward/back arrows were clicked.
  // In those cases it shows the reference that was than recorded, a while ago.
  // This prevents the trackers from recording the previously recorded reference again.
  string recorded_reference;
  if (reference_pointer >= 0) {
    recorded_reference = reference (recorded_books[reference_pointer], recorded_chapters[reference_pointer], recorded_verses[reference_pointer]);
  }
  if (current_reference == recorded_reference) {
    time_elapsed = 0;
    return;
  }
  time_elapsed++;
  if (time_elapsed >= 5) {	// The number of seconds after which the reference gets recorded.
    recorded_reference = current_reference;
    recorded_books.push_back(book);
    recorded_chapters.push_back(chapter);
    recorded_verses.push_back(verse);
    // When a new reference is recorded, the pointer goes straight to the end of the list.
    // Even if it was not pointing at the end after the forward/back buttons were clicked.
    reference_pointer = recorded_books.size() - 1;
  }
  if (current_reference != previous_reference) {
    previous_reference = current_reference;
    time_elapsed = 0;
  }
}


void Track::get_next_reference (string& book, string& chapter, string& verse, bool next) {
  if (next) {
    reference_pointer++;
  } else {
    reference_pointer--;
  }
  book = recorded_books[reference_pointer];
  chapter = recorded_chapters[reference_pointer];
  verse = recorded_verses[reference_pointer];
}


bool Track::next_reference_available() {
  return (reference_pointer < int(recorded_books.size() - 1));
}


bool Track::previous_reference_available() {
  return (reference_pointer > 0);
}


string Track::reference (const string& book, const string& chapter, const string& verse) {
  return book + " " + chapter + ":" + verse;
}


void Track::clear() {
  time_elapsed = 0;
  reference_pointer = -1;
  previous_reference.clear();
  recorded_books.clear();
  recorded_chapters.clear();
  recorded_verses.clear();
}

