/*
** Copyright (C) 2003 The Free Software Foundation.
** ** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef INCLUDED_UNDO_H
#define INCLUDED_UNDO_H

#include "libraries.h"


class Undo {
 public:
  Undo(int dummy);
  ~Undo();
  void clear();
  void on_insert(const Gtk::TextBuffer::iterator& pos, const Glib::ustring& text, int bytes);
  void on_erase(const Gtk::TextBuffer::iterator& range_begin, const Gtk::TextBuffer::iterator& range_end);
  void undo(Glib::RefPtr<Gtk::TextBuffer> buffer);
  void redo(Glib::RefPtr<Gtk::TextBuffer> buffer);
  bool can_undo();
  bool can_redo();
 private:
  /*
  The on_insert callback gives which text was inserted, and where, 
  but the erase callback does not indicate how much text was erased.
  Therefore we need a shadow string, 
  so that the information can be deduced from comparing the string 
  with the current text in the TextBuffer.
  */
  Glib::ustring shadow;
  vector<int> undo_position;
  vector<Glib::ustring> undo_text;
  vector<bool> undo_insert;
  void stack_operation(int position, const Glib::ustring& text, bool insert);
  bool non_undoable_action;
  unsigned int pointer;
};


#endif

