/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "usfm.h"


Glib::ustring usfm_extract (Glib::ustring& line) {
  /*
    This returns the usfm from the line, e.g. \id.
    The usfm is removed from the line.
   */
  Glib::ustring s = trim(line);
  Glib::ustring returnvalue = "";
  if (s.length() > 1) {
    if (s.substr(0,1) == "\\") {
      unsigned int endposition = s.find_first_of(" \\", 1);
      if (endposition == string::npos) {
	line = "";
	returnvalue = s;
      } else {
	returnvalue = s.substr(0, endposition);
	line.erase(0, endposition);
	line = trim(line);
      }
    }
  }
  return returnvalue;
}


Glib::ustring usfm_extract_marker (Glib::ustring& line) {
  // Returns the usfm marker from the line, but without
  // the first backslash, so e.g. id.
  // The backslash and the usfm marker are removed
  // from the line.
  Glib::ustring returnvalue = usfm_extract(line);
  if (returnvalue.length() > 0) returnvalue.erase(0,1);	// Remove backslash.
  return returnvalue;
}


int usfm_search_marker_from (const Glib::ustring& line, unsigned int from) {
  /* 
     Searches for any usfm (whether it exists or not).
     It starts searching at position "from".
     It returns the position where it found the usfm, or
     string::npos is not found.
   */
  return line.find("\\", from);
}


unsigned int usfm_search_marker_from_length (const Glib::ustring& line, unsigned int from) {
  /*
    Returns the length of the usfm that is found at position
    "from".
  */
  Glib::ustring s;
  s = line.substr(from, line.length());
  return usfm_extract(s).length();
}


string get_id_from_file(const string& filename) {
  ReadText rt (filename, true);
  unsigned int max = 10;
  if (max >= rt.lines.size())
    max = rt.lines.size();
  Usfm usfm(0);
  for (unsigned int i = 0; i < max; i++) {
    Glib::ustring line = trim(rt.lines[i]);
    Glib::ustring marker = usfm_extract_marker(line);
    if (usfm.is_id(marker)) {
      try {
        return line.substr(0, 3);
      }
      catch (exception& ex) {
        cerr << ex.what() << endl;
      }
    }
  }
  return "";
}
